// Copyright (c) 2014  Albrecht Weinert, Bochum *
// $Id: cli.h 2 2017-01-25 16:49:03Z albrecht $

/**  \addtogroup weAutSys
 * @{
 *    \addtogroup appLaySup
 * @{
 *     \addtogroup shellCLI
 * @{
 *
 * \file cli.h
 *
 *  \ref intro_sec "weAutSys"' command line interpreter (CLI)
 *
 *  This file contains the definitions for \ref intro_sec "weAutSys"' system
 *  threads and functions for the command line interpretation.
 *  Please find detailed description of the variables, functions etc.
 *  in the module  \ref shellCLI "Command line interpreter" and also
 *  at \ref sysThr "system threads".
 *
 *  This is system software and must not be modified for user or
 *  application programs.
 *
 *  This file is part of \ref intro_sec "weAutSys" &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2014 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp;
 *                            <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 2 $
 * $Date: 2017-01-25 16:49:03 +0000 (Wed, 25 Jan 2017) $
 */
//  SVN last change by $Author: albrecht $

/** @} */  // \addtogroup shellCLI
/** @} */  // \addtogroup appLaySup

#ifndef __CLI_H__
#define __CLI_H__

#include "pt/pt.h"
#include "we-aut_sys/common.h"

/**
 * \addtogroup appLaySup
 * @{
 * \addtogroup shellCLI
 * @{
 */

/** The user / application CLI registration type  */
struct appCLIreg_t {

/** The user CLI thread's function (pointer to)
 *
 *  The function pointed to must implement a user / application thread
 *  (behaviour) according to all Protothreads and \ref intro_sec "weAutSys" rules therefore.
 *
 *  This being NULL means there is no user command line interpreter (CLI).
 */
   p2ptFunA fun;

/** Array of the (flash) user software command definitions
 *
 *  This flash array of flash texts must be defined and initialised
 *  by user / application software. It may be initialised as NULL
 *  if the user software chooses to define / implement no own CLI commands.
 *
 *  Otherwise \ref helpUserCm[] should be the first entry followed by
 *  (flash text) command definitions. The last entry must always be NULL like
 *  in the example: \code
  char* userCommands[]) = { helpUserCm, // 0 no command just separator
      comADinputs, comADoff, // 1 2     + INDEX_OFFSET_LIST2
      comDiscoBun, comDemCount, // 3 4  + INDEX_OFFSET_LIST2
      NULL};  \endcode
 *
 *  Index 0 (the first entry) will never be used or executed as command. It is
 *  to be used as the user commands headline for help (command list) output.
 *
 *  It is also essential that all command definitions made by user software
 *  are in flash memory (by \ref INFLASH macro) the example: \code
  INFLASH(char const comDiscoBun[]) = "discoBunny [t/10ms] Demo is disco lights \n";
  INFLASH(char const comDemCount[]) = "countDemo  [t/10ms] Demo is DO count     \n";
 *  \endcode
 *  \sa INDEX_OFFSET_LIST2
 *  \sa FOLLOW_UP
 */
   char const * const * userCommands;
}; // appCLIreg_t

/** The user / application command line interpreter (CLI) registration
 *
 *  Command line interpretation can occur in multiple contexts like UART
 *  link, multiple connections to Telnet server etc. And it can occur
 *  "simultaneously" in in any of said contexts.
 *
 *  Nevertheless the application CLI (if any) is
 *  \ref registerAppCli "registered" here for once. This structure with this
 *  registration may and will be copied to the state / structure of said
 *  usages / connections.
 *
 *  It may take some time and (disconnect / connect) events for a changed
 *  application CLI \ref registerAppCli "registration" to take effect. Or,
 *  in other words dynamic changes of an application CLI aren't supported
 *  yet. It is strongly recommended for the user software to have zero or
 *  one application CLI implementation and register it at
 *  \ref appInitThreadF "reset start-up" (phase 4).
 */
extern struct appCLIreg_t appCLIreg;

/** Register a user / application command line interpreter (CLI)
 *
 *  The user CLI thread function and the user commands description will be
 *  registered.
 *
 *  To de-register call with at least one parameter NULL.
 *
 * \param threadF the user CLI thread function to register
 * \param userCommands flash array of flash strings of user commands
 *                     (+ explanations). The (non NULL) array must end with
 *                      NULL entry.
 * \sa appCliThreadF
 * \sa appInitThreadF
 */
void registerAppCli(p2ptFunA threadF, char const * const userCommands[]);

/** Initialise a thread (structure)  as  command line interpreter (CLI) thread
 *
 *  This function initialises the common (union) part of the structure
 *  supplied as a command line interpreter (CLI) thread for the
 *  usage of a stream (\c file) for output and the
 *  \ref registerAppCli "registered" application command line interpreter
 *  (CLI) thread function.
 *
 *  The (CLI) thread state will completely be forced to empty initial state
 *  with no command and line set.
 *
 * \param thread the thread structure to initialise
 * \param file  The streams to be used for output / reply; NULL is OK when
 *              all command may be executed (or even fail) silently
 * \sa appCliThreadF
 * \sa appInitThreadF
 */
void initAsCLIthread(struct thr_data_t * thread, FILE * file);

/** Set a command line in the command line interpreter (CLI) thread data
 *
 *  The given \c line will be set in the \c cliThread, the command (i.e. the
 *  first) token will be determined and the system and user CLI
 *  thread state will be forced to empty initial state.
 *
 *  Do not call this function while system or user CLI threads are active.
 *
 *  The \c line (buffer) is copied to have it as thread state. The copying
 *  and extra memory usage is avoided if the parameter \c line is prepared
 *  in and supplied as \c cliThread->cliThrData.line .
 *
 * \param  cliThread pointer to the CLI thread to interpret line
 * \param  line the command line (pointer to 0-terminated RAM string)
 * \param  length \c line's length
 * \return the command number
 * \sa appCliThreadF
 * \sa appInitThreadF
 */
uint8_t setCliLine(struct cliThr_data_t * const cliThread,
                                        char * const line, uint16_t length);

/** The user / application specific command line interpreter thread
 *
 *  If something is to be done in the user / application software for
 *  (human readable) command lines (set by \ref setCliLine()) a command line
 *  interpreter protothread thread function must be provided therefore.
 *
 *  This has to be done (according to this declaration) in an application
 *  specific source file and that thread function has to be
 *  \ref registerAppCli "registered".
 *
 *  This thread function will be scheduled automatically if and as long as
 *  a user command is pending. If an output stream belongs to the calling
 *  context the standard streams will have been switched to it space for
 *  40 characters is guaranteed. So this user thread function's implementation
 *  will not have to worry about those house keeping tasks.
 *
 *  This thread function will be re-scheduled on a pending user command
 *  as long as the command (\c cliThread->commNumb ) is not set to 0 or this
 *  function does not \ref PT_ENDED "ends" or \ref PT_EXITED "exits".
 *
 *  \sa initAsCLIthread
 *  \sa appInitThreadF
 *  \sa sysCliThreadF
 */
ptfnct_t appCliThreadF(struct cliThr_data_t * cliThread);

/** The system command line interpreter thread
 *
 *  If a (human readable) command line has been set by \ref setCliLine()
 *  this thread might be scheduled (until end / exit) to execute a parsed
 *  system command.
 *
 *  If user commands and a user CLI thread (function) are
 *  \ref registerAppCli "registered" this thread will also be scheduled if a
 *  user command was parsed. This thread function will delegate all user
 *  command execution to the \ref registerAppCli "registered" user
 *  user CLI thread (function)
 *
 *  If no (unambiguous) command was found a report will be made if allowed so
 *  in the \c cliThread structure.
 *
 *  \sa initAsCLIthread
 *  \sa appInitThreadF
 *  \sa appCliThreadF
 *  \return \ref PT_YIELDED if waiting for conditions that may become
 *           automatically true; \ref PT_WAITING if blocked by conditions
 *           that other's action may set true;  \ref PT_EXITED if stopped
 *           by conditions that never will become true (sorry, not all work
 *           done and never will; \ref PT_ENDED if ready with all work
 */
ptfnct_t sysCliThreadF(struct cliThr_data_t * cliThread);

/**  Report the use of an un-implemented option for a command
 *
 *  This is a helper function for the implementation on CLIs (command line
 *  processors) only. This function requires stdout connected to CLI streams
 *  and enough output space available in the respective stream.
 *
 *  \param  cliThread pointer to the CLI (and hence the thread busy
 *          to execute actual "command -wrongOpt)
 */
void unimplOptionReport(struct cliThr_data_t * cliThread);

/** Wait (yielding) for command execution to end
 *
 *  This macro waits yielding the main (input, application) thread for all
 *  command interpretation to end. It is a helper for a user defined
 *  \ref appSerInpThreadF "serial input thread".
 *
 *  \param appThread pointer to the (complete, type \ref thr_data_t)
 *         thread structure
 */
#define YIELD_FOR_BUSY_CLI(appThread)  \
		do { \
		   PT_YIELD_FLAG = 1; \
			if (!(appThread)->cliThrData.commNumb) break; \
		   PT_YIELD_FLAG = 0; \
			LC_SET((appThread)->pt.lc);  \
			const uint8_t cliRet = sysCliThreadF(&(appThread)->cliThrData); \
			if (cliRet == PT_YIELDED) return PT_YIELDED; \
         /*if (cliRet == PT_EXITED) (appThread)->cliThrData.commNumb = 0;*/ \
		} while(0)



/** @} */ // Doxygen addtogroup shellCLI
/** @} */ // Doxygen addtogroup appLaySup

#endif /* __SYST_THREADS_H__ */
/** @} */ // Doxygen addtogroup weAutSys
