// Copyright (c) 2011 Albrecht Weinert, Bochum
// $Id: proc_io.h 2 2017-01-25 16:49:03Z albrecht $


/**
 * \addtogroup weAutSys
 * @{
 *
 * \addtogroup procIO
 * @{
 *
 * \file proc_io.h
 *
 *  \ref intro_sec "weAutSys"'/weAut_01's (low level) system calls and services
 *           for process I/O and HMI
 *
 *  This file contains the definitions for \ref intro_sec "weAutSys" /
 *  weAut_01  (low level) system calls and services for process input and
 *  output (I/O) and I/O for the human machine interface (HMI). This is
 *  system software and must not be modified for user or application programs.
 *
 * This file is part of \ref intro_sec "weAutSys" &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp;
 *                            <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 2 $
 * $Date: 2017-01-25 16:49:03 +0000 (Wed, 25 Jan 2017) $
 */
//  SVN last change by $Author: albrecht $

/** @} */ // Doxygen addtogroup   procIO

#ifndef PROC_IO_H
#define PROC_IO_H

#include <avr/io.h>
#include <avr/interrupt.h>

/**
 * \addtogroup procIO
 * @{
 * \addtogroup hmiIO
 * @{
 */

/** Output to DI display LEDs
 *
 *  This function outputs an eight bit value to the process digital input (DI)
 *  status LEDs.
 *
 *  On  \ref intro_secH "weAut_01" (weAut_00) this is the complete SPI 1
 *  output function to the DI-LED driver for its 8 green status LEDs (in
 *  neighborhood of the DI clamps).
 *
 *  Hint: Using the SPI interface is avoided if the result can be obtained
 *  by output enabling / disabling the LED driver. This (doubtful)
 *  optimisation allows (regular) blinking even if the SPI interface is
 *  blocked for other operations. This feature may be removed in later
 *  hardware versions.
 *
 *  On other platforms (arduMega2560 and the like) the value will be output
 *  to an eight bit port configured therefore. If no DI-LED port is configured
 *  (DI_LEDs0_PORT undefined) no output will be done (but the
 *  \ref actDiLEDs "states" are hold).
 *
 *  \param value 8 bits for 8 green LEDs, upper row, located same column
 *         as the corresponding DI  clamps
 *  \sa   toDOdriver
 */
void toDiLEDs(uint8_t value);

/** DI display LEDs: actual value
 *
 *  This function returns the actual value of the DI LEDs.
 *
 *  \sa   toDiLEDs()
 */
uint8_t actDiLEDs(void)  __attribute__((always_inline));

/** DI display LEDs: previous value
 *
 *  This function returns the actual value of the DI LEDs.
 *
 *  \sa   toDiLEDs()
 */
uint8_t prvDiLEDs(void)  __attribute__((always_inline));

/** Turn DI display LEDs off
 *
 *  If the LEDs are off (==0) nothing is done.<br />
 *  If at least one is on this is a shorter equivalent for \c toDiLEDs(0).
 *  \sa   toDiLEDs
 *  \sa   onDiLEDs
 */
void offDiLEDs(void)  __attribute__((always_inline));

/** Turn DI display LEDs on
 *
 *  If not all LEDs are off (!=0) nothing is done.<br />
 *  If they are off this function is a (quick) equivalent for
 *  \c toDiLEDs( prvDiLEDs()).
 *
 *  \sa   offDiLEDs
 */
void onDiLEDs(void) __attribute__((always_inline));

/** @} */ // addtogroup hmiIO

//--------------------------------------------------

/**
 * \addtogroup digOut
 * @{
 */


/** Output to digital (process) output DO
 *
 *  This function outputs an eight bit value to the standard digital process
 *  output.
 *
 *  On  \ref intro_secH "weAut_01" (weAut_00) this is the complete SPI 1
 *  output function to the protected DO driver and its 8 green (ordered
 *  output) status LEDs.
 *
 *  The \c value goes direct to the 8 green DO LEDs. The state driven to the
 *  DO outputs further depends on the state of the driver (enable / disable,
 *  fault) and on the level of its supply -- the load voltage (LV) or
 *  "Lastspannung" in German.
 *
 *  On other platforms (arduMega2560 and the like) the value will be output
 *  to the eight bit port configured. If no DO is configured (marcro DO_0_PORT
 *  undefined) no output will be done (but the \ref actDOdriver "states" are
 *  hold).
 *
 *  \param value 8 bits for 8 green LEDs, upper row, located in the same
 *         column as the corresponding DO  clamps
 *  \sa   enableDOdriver
 *  \sa   lowLV
 *  \sa   toDiLEDs
 */
void toDOdriver(uint8_t value);

/** DO digital (process) output: actual value
 *
 *  This function returns the actual value of the DI LEDs.
 *
 *  \sa   toDOdriver
 */
uint8_t actDOdriver(void)  __attribute__((always_inline));

/** DO digital (process) output: previous value
 *
 *  This function returns the actual value of the DI LEDs.
 *
 *  \sa   toDOdriver
 */
uint8_t prvDOdriver(void)  __attribute__((always_inline));

/** Enable the digital (process) output DO driver
 *
 *  This function allows all 8 DO output channels to be turned off
 *  respectively on.
 *
 *  On \ref intro_secH "weAut_01" (weAut_00) this will control the protected
 *  DO driver (9..28V "Lastspannung") and does in no way influence the output
 *  value set by \c toDOdriver(uint8_t), which will be still displayed by the
 *  (8 green) DO LEDs.
 *
 *  The off state will be signaled by the (red) DO disable LED.
 *
 *  Also, turning off is the procedure to reset any output driver's fault
 *  state, be it on over temperature or over current. Turning off on a fault
 *  state will probably not make the situation worse. A fault condition
 *  turns the output driver off anyway.
 *
 *  On other platforms this will be used to mask out the the output value set
 *  by \c toDOdriver(uint8_t) respectively re-enable it.
 *
 *  \sa   toDOdriver
 *  \param state true: enable; false: disable
 */
void enableDOdriver(uint8_t state)  __attribute__((always_inline));

/** Digital (process) output DO driver enabled
 *
 *  \sa   toDOdriver
 *  return state true: enabled; false: disabled
 */
uint8_t doDriverEnabled(void)  __attribute__((always_inline));

/** Digital (process) output DO driver OK
 *
 *  On  \ref intro_secH "weAut_01" (weAut_00) this function returns true if
 *  the protected DO driver (9..28V "Lastspannung") is neither in over current
 *  nor over temperature shut down state for at least one output channel.
 *
 *  On other platforms with a Port dedicated to DO this function returns true
 *  if the re-read out pin values equal the actual output. If that is not so
 *  there may be shorts to Vdd or Gnd or configuration problems.
 *
 *  In all other cases this function blindly returns true.
 *
 *  \sa   toDOdriver
 *  return state true: OK; false: possible problems as described
 */
uint8_t doDriverOK(void)  __attribute__((always_inline));


/** @} */ // addtogroup digOut

//--------------------------------------------------

/**
 * \addtogroup procIn
 * @{
 */


/** Digital (process) input DI (direct)
 *
 *  This function returns the (8) digital inputs in one byte.
 *  The value by this function will be the immediate un-filtered read.
 *  Hence it is a low level function rarely (not to say never) to be used by
 *  user /application software directly.
 *
 *  For configurations without such inputs 0 will be returned. Channels
 *  used (in weAut_01 e.g.) for analogue input and disabled for digital
 *  reading (by DIDR0 the digital input disable register) will also return
 *  0 in the respective bit position.
 */
uint8_t dctDI(void)  __attribute__((always_inline));

/** Digital (process) input DI (system implementation)
 *
 *  Must not to be used by user / application software.
 */
void procDIcyc(void);


/** Actual digital (process) input DI
 *
 *  This function returns the (8) digital inputs in one byte.
 *  The value by this function will be the actual un-filtered read, done by
 *  system software within the last one millisecond. Hence, the value is
 *  0..1 ms old.
 *
 *  Hint: This statement is true for the normal configuration where all
 *  DI processing is done in the 1 ms system thread. If done more seldom by
 *  system (re-) configuration, apply the appropriate factor to all filter
 *  and timing values given for this function and the other related DI
 *  functions.
 *
 *  Hint 2: \c actDI(), \c prvDI(), \c lbpDI() and \c dctDI() are low level
 *  function rarely to be used by user / application software. Therefore
 *  \c filDI() is recommended.
 *
 *  \sa prvDI
 *  \sa lbpDI
 *  \sa filDI
 */
uint8_t actDI(void)  __attribute__((always_inline));

/** Previous digital (process) input DI
 *
 *  This function returns the (8) digital inputs in one byte.
 *  The value here will be the previous un-filtered read, done by system
 *  software before the actual read. The value will be 1 ms older than
 *  \c actDI(), i.e. 1..2 ms old.
 *
 *  \sa actDI
 *  \sa lbpDI
 *  \sa filDI
 */
uint8_t prvDI(void)  __attribute__((always_inline));

/** Last before previous digital (process) input DI
 *
 *  This function returns the (8) digital inputs in one byte.
 *  The value here will be the last before previous un-filtered read.
 *  The value will be 2 ms older than \c actDI().
 *
 *  \sa actDI
 *  \sa prvDI
 *  \sa filDI
 */
uint8_t lbpDI(void)  __attribute__((always_inline));

/** The final or filtered digital (process) input DI
 *
 *  This function returns the (8) digital inputs in one byte.
 *  The value here will be the final value reflecting state values stable
 *  over the last three milliseconds by appropriate filter algorithm.
 *
 *  A stable input change will be reflected here after 2 .. 3 ms.
 *
 *  It is this (stable) value that is optionally used to enlarge the
 *  hysteresis (by slightly shifting the input thresholds),
 *
 *  \sa actDI
 *  \sa prvDI
 *  \sa dctDI
 *  \sa upDIthresh4hyst
 */
uint8_t filDI(void)  __attribute__((always_inline));


/** Shift DI thresholds up mask permanently
 *
 *  For bits set 1 in this mask the respective DI channels
 *  permanently get higher input thresholds compared to those
 *  without such settings.
 *
 *  High settings: about 11V on; about 9V off.<br />
 *  Low settings:  about 6V on; about 3V off.<br />
 *
 *  This DI level setting works with \ref intro_secH "weAut_01" (weAut_00)
 *  only. Boards without "real" digital process input have only CMOS logic
 *  inputs.
 *
 *  \sa upDIthresh4hyst
 *  \sa filDI
 */
extern uint8_t upDIthreshForce;

/** Shift DI thresholds up mask conditionally / larger hysteresis
 *
 *  For bits set 1 in this mask byte the respective DI channels
 *  permanently get higher input thresholds on the condition of
 *  their \c fiDI() value being 1. This effectively widens this DI channels
 *  hysteresis.
 *
 *  Wide setting: about 11V on; about 3V off.
 *
 *  The hysteresis widening will only work if the respective bit in
 *  \c upDIthreshForce is <b>not</b> set.
 *
 *  This DI level / hysteresis settings work with \ref intro_secH "weAut_01"
 *  (weAut_00) only. Boards without "real" digital process input have only
 *  CMOS logic inputs.
 *
 *  \sa upDIthreshForce
 *  \sa filDI
 */
extern uint8_t upDIthresh4hyst;

/** Use channel(s) as AI instead of DI
 *
 *  For bits set 1 in this mask byte the respective input channels are
 *  uses as analogue inputs AI (en lieu de DI).
 *
 *  This variable must not be set directly but changed by using
 *  \c setAIchannels.
 */
extern uint8_t aiChannels;

/** Set the usage of channel(s) as AI instead of DI
 *
 *  For bits set 1 in mask the respective input channels are
 *  uses as analogue inputs AI (en lieu de DI).
 *
 *  This DI/AI configuration is only relevant with \ref intro_secH "weAut_01"
 *  (weAut_00) where potential AI and DI share the same (rare) protected
 *  input clamps. Boards with direct access to (an abundance of) Controller
 *  pins won't use that feature.
 *
 *  \sa  aiChannels.
 */
void  setAIchannels(uint8_t mask);

/** Analogue input available
 *
 *  System software will set a bit if an analogue conversion, enabled by
 *  \c aiChannels, is complete. Converting all 8 channels (if told so by
 *  \c aiChannels) will take about 700s.
 */
extern uint8_t aiConvd;


/** Analogue input results
 *
 *  This array contains the latest AI conversion results (only for those
 *  channels enabled by \c aiChannels).
 *
 *  The results are restricted to 8 bit resolution which is quite appropriate
 *  regarding the use of weAut_01's use of its inputs designed as protected
 *  DIs in an analogue mode.
 */
extern uint8_t aiResult[8];


/** @} */ // addtogroup procIn

//--------------------------------------------------

/**
 * \addtogroup  hmiIO
 * @{
 */


/** Enter key is released
 *
 *  This is the direct un-filtered state of the enter button (weAut_01). <br />
 *  0 (false) is returned if the button is pressed. <br />
 *  Non 0 is returned if the enter button is released.
 *
 *  On the \ref intro_secH "weAut_01" board the button nearer to the Ethernet
 *  and serial connectors is (hard) reset). The one more far away is the
 *  "enter" button handled here.
 *
 *  Configurations without that key always return non 0 (true).
 */
#if defined(weAut_01) || defined(weAut_00) || defined(arduMega2560)
#  define enterKeyRel(X) (PINB & 1)
#else
#  define enterKeyRel(X)  (1)
#endif

/** Filtered Port B input
 *
 *  Port B0 == 1 means  enter key on (weAut_01) is released.
 *  \sa enterKeyReleased
 *  \sa enterKeyPressed
 */
   extern uint8_t  pbFil;

/** Enter key is (stable) released
 *
 *  This is the filtered state of the enter key
 *  (\ref intro_secH "weAut_01").<br />
 *  0 (false, OFF) is returned if the key is pressed. <br />
 *  Non 0 is returned if the enter key is released.
 *
 *  Configurations without that key always return 1 (non 0).
 *
 * \sa enterKeyPressed
 */
#if defined(weAut_01) || defined(weAut_00) || defined(arduMega2560)
#  define enterKeyReleased(X) (pbFil & 1)
#else
#  define enterKeyReleased(X)  (1)
#endif

/** Enter key is (stable) pressed
 *
 *  This is the filtered state of the enter key (weAut_01) .<br />
 *  Non 0  (true, ON) is returned if the key is released. <br />
 *  0 (false, OFF) is returned if the enter key is pressed.<br />
 *
 *  Configurations without that key always return 0.
 *
 *  \sa enterKeyReleased
 */
#if defined(weAut_01) || defined(weAut_00) || defined(arduMega2560)
#  define enterKeyPressed(X) (!(pbFil & 1))
#else
#  define enterKeyPressed(X)  (0)
#endif


/** @} */ // addtogroup hmiIO

//--------------------------------------------------

/**
 * \addtogroup  digOut
 * @{
 */

/** Load voltage low
 *
 *  Non 0 means a low load voltage supply (LV, German Lastspannung). In that
 *  case the digital output's (DO's) supply is below about 10.3V. This is
 *  also approximately the level where weAut_01's lower blue LEDs shine with
 *  only medium brightness. Lower side of the board means the RJ45 jack side.
 *
 *  At 10 V load voltage the processor's an all other periphery's supply is
 *  still guaranteed, even if the redundant supply should be absent. But the
 *  operation of 12V periphery is probably at risk and that of 24V
 *  periphery normally gone already.
 *
 *  Hint 1: If a more accurate control of LV is needed, its possible to tie
 *  an (DI) input as AI to LV (using it in the analogue mode then).
 *
 *  Hint 2: weAut_01 can have this LV threshold risen to (industrial
 *  automation) 24V mode by a simple wire bridge. See the hardware manual
 *  of your module.
 *
 *  Hint 3: \c lowLV is implemented as a counter counting up to 128
 *  respectively down to 0 every millisecond according to LV > about 10,3V.
 *  The 0-1-transition will set / un-set the red (test) LED, so red = on
 *  will indicate "LV low". This may slightly interfere with other usages
 *  of this red test LED during development / debugging phases.<br />
 *  After an intermediate use of the red (test) LED it can be restored to
 *  the proper (LV) state by \c setStatusLedRd(lowLV).
 *
 *  Hint 4: This variable should never be set by user / application software.
 *
 *  This feature is available at \ref intro_secH "weAut_01" (weAut_00)
 *  only. Boards without "real" process signals and load supply handling can't
 *  have it.
 *
 *  \sa setStatusLedRd
 */
   extern uint8_t  lowLV;

/** @} */ // addtogroup digOut

#endif /* PROC_IO_H */

/** @} */ // Doxygen addtogroup  procIO (Low level process I/O and HMI)
/** @} */ // Doxygen defgroup weAutSys
