/*
 * Copyright (c) 2011 Albrecht Weinert, Bochum
 *
 * This file is part of \ref intro_sec "weAutSys".
 *
 * Author: Albrecht Weinert <albrecht@a-weinert.de>
 *
 * $Id: timing.h 2 2017-01-25 16:49:03Z albrecht $
 */

/**
 * \addtogroup weAutSys
 * @{
 * \addtogroup sysSer
 * @{
 * \addtogroup timeKeep
 * @{
 */

/** \file timing.h
 *
 *  \ref intro_sec "weAutSys"  definition of timing services
 *
 *  This is system software and must not be modified for user or
 *  application programs.
 *
 * This file is part of \ref intro_sec "weAutSys" &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp;
 *                            <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 2 $
 * $Date: 2017-01-25 16:49:03 +0000 (Wed, 25 Jan 2017) $
 */
//  SVN last change by $Author: albrecht $

/** @} */ // Doxygen addtogroup timeKeep
/** @} */ // Doxygen addtogroup sysSer

#ifndef _TIMING_H_
#define _TIMING_H_

#include "maketime_utc.h"
#include "we-aut_sys/common.h"

/**
 * \addtogroup sysSer
 * @{
 * \addtogroup timerHdl
 * @{
 */

//------------------   Milliseconds time ------------------------------

/** Milliseconds since power up
 *
 *  The value returned is the basis of all \ref intro_sec "weAutSys" (relative) timing in
 *  milliseconds resolution. It is the 32 bit unsigned number of milliseconds
 *  since system start or reset. The value will wrap around after a bit more
 *  than 49 days.
 *
 *  A modulo arithmetic compare function (\ref geU32ModAr()) used for timers
 *  / time values of that type will give an correct >= answer even over that
 *  wrap around provided the the difference is less than half the above
 *  range. <br />
 *  Hence timers for about 24,8 days can be made with milliseconds
 *  resolution.
 *
 *  Monotony in the sense of modulo 2**32 (or \ref geU32ModAr())
 *  is guaranteed, but there may be gaps (little leaps).
 *
 *  \sa ucnt32_t
 */
uint32_t msClock(void)  __attribute__((always_inline));


/** Milliseconds per minute
 *
 *  \sa MSEC_HOUR
 *  \showinitializer
 */
#define MSEC_MINUTE 60000

/** Milliseconds per hour
 *
 *  \sa MSEC_MINUTE
 *  \showinitializer
 */
#define MSEC_HOUR 3600000

/** Milliseconds per day
 *
 *  This value can be used as milliseconds timer period for every day
 *  repetitive actions like closing or opening windows, shutters or doors.
 *
 *  \note Keep in mind that milliseconds timers will fail for periods longer
 *  than 28 days. Consider using seconds based timers.
 *
 *  \sa MSEC_MINUTE
 *  \showinitializer
 */
#define MSEC_DAY 86400000

/** Milliseconds per week
 *
 *  This value can be used as period for milliseconds timers of one week.
 *  Often unused aggregates, like pumps, are run for a short time every
 *  week to be kept movable.
 *
 *  \note Keep in mind that milliseconds timers will fail for periods
 *  longer than 4 weeks. Consider using seconds based timers.
 *
 *  \sa MSEC_MINUTE
 *  \hideinitializer
 *  \sa SECONDS_IN_WEEK
 */
#define MSEC_WEEK 604800000


/** Seconds in a day
 *
 *  This is the divisor used to split second based absolute time into
 *  a) days since base date and b) seconds since midnight.
 *
 *  This value can be used as period for seconds based timers.
 *  \showinitializer
 */
#define SECONDS_IN_DAY 86400

/** Seconds in a week
 *
 *  This value can be used as period for seconds based timers.<br />
 *  Often unused aggregates, like pumps, are run for a short time every
 *  week to be kept movable.
 *  \hideinitializer
 */
#define SECONDS_IN_WEEK 604800

/** Seconds in a (normal) year
 *
 *  This value is the equvalent of 365 days.
 *  \hideinitializer
 */
#define SECONDS_IN_YEAR 31536000

/** The system ms counter / clock value for running / system up time
 *
 *  The value is in the range 0..999 and will be incremented respectively
 *  wrapped around modulo 1000 by system software (i.e. in the
 *  system 1ms thread). Wrapping means incrementing the
 *  \ref secClock() (i.e. secDurTimers.end.v32).
 *
 *  Monotony (in modulo 1000 terms) is guaranteed, but there may be
 *  gaps.<br />
 *  This value is not affected by absolute (local) date / time settings or
 *  corrections.
 *
 *  Never to be modified elsewhere.
 *  \sa  msAbsClockCount
 */
extern uint16_t msSystClockCount;

/** The system ms clock value for local time
 *
 *  The value is in the range 0..999 and will be incremented respectively
 *  wrapped around modulo 1000 by system software (i.e. in the
 *  system 1ms thread). Wrapping means incrementing all seconds resolution
 *  clock values, like \ref secTime308Loc() etc., but not \ref secClock().
 *
 *  Monotony (in modulo 1000 terms) is guaranteed except for time / date
 *  settings corrections.
 *
 *  Never to be modified elsewhere.
 *  \sa  msSystClockCount
 */
extern uint16_t msAbsClockCount;

/**
 * @}
 * \addtogroup timeKeep
 * @{
 */

//------------------   Seconds based time and date  -------------------

/** The system's run time in seconds
 *
 *  The value will be handled (incremented) by system software (i.e. the
 *  system's 1s thread). Monotony is guaranteed. There may be gaps, but that
 *  is very improbable &mdash; all miss-behaving (application) software
 *  leading to this should get a watchdog reset.
 *
 *  Do not modify the underlying variable! See also the
 *  \ref do_not_mod_warn "warning".
 *
 *  \sa msClock(void)
 *  \sa msSystClockCount
 */
uint32_t secClock(void) __attribute__((always_inline));

/** Seconds since March 1st 2008 local time
 *
 *  This gives the local time in seconds since (Saturday) March 1st 2008 in
 *  local time &mdash; i.e. with zone offsets and
 *  \ref isDST "summertime (DST)" shifts.
 *
 *  \sa  secClock()
 *  \sa msAbsClockCount
 */
#define secTime308Loc(x) (secDatTimers.end.v32)

/** The local time as FAT time
 *
 *  The local time contains zone offsets and \ref isDST "summertime (DST)"
 *  shifts. This function returns this time in a 32 bit (uint32_t) packed
 *  structure used for the traditional FAT file systems. Note that this
 *  (old DOS) time format has a 2s resolution only. The format is:
 *
 *  bit 31..25: year from 1980 (0..127; 2012 is 32) <br />
 *  bit 24..21: month (1..12) <br />
 *  bit 20..16: day in month (1..31) <br />
 *  bit 15..11: hour (0..23) <br />
 *  bit 10...5:&nbsp; minute (0..59) <br />
 *  bit &nbsp;4...0: &nbsp; second / 2 (0..29, 30 if leap second)
 *
 *  \sa secTime308Loc
 */
uint32_t getFATtime(void);

//uint32_t secTime308Loc(void) __attribute__((always_inline));

/** Count of secTime308Loc adjustments
 *
 *  Significant \ref secTime308Loc "local time" adjustments are counted
 *  here. 0 means no time date setting since reset (hence usually
 *  incorrect). <br />
 *  Counting wraps from 255 to 1.
 *
 *  Significant means hard setting of \ref secTime308Loc or its ms (0.999)
 *  fraction part \ref msAbsClockCount. Adjustments by variable controlled
 *  oscillator (VCO) are not counted.
 */
extern uint8_t adjustUTCcount;


/** Seconds since March 1st 2008 UTC
 *
 *  The value will be the time in seconds since
 *  \ref intro_sec "weAutSys"'s zero date March 1st 2008 UTC.
 *
 *  UTC here means without any zone or \ref isDST "DST" offsets. <br />
 *  The returned value can easily be converted to
 *  \ref secTimeUnixUTC() "UNIX" or \ref secTimeNtpUTC() "NTP" time.
 *
 *  \sa  secTime308Loc()
 */
uint32_t secTime308UTC(void) __attribute__((always_inline));


/** Convert to seconds since January 1st 1970 UTC (Unix time)
 *
 *  The returned value will be time in seconds since the UNIX
 *  time start date.
 *
 *  \param secTime308UTC the time in seconds since March 2008 UTC
 *  \sa  secClock()
 *  \sa secTime308UTC()
 */
uint32_t secTimeUnixUTC(uint32_t secTime308UTC)  __attribute__((always_inline));

/**  Convert to seconds since January 1st 1990 UTC (NTP time)
 *
 *  The returned value is the time in seconds since the NTP
 *  time start date. Called with \ref secClock() as parameter \c stamp this
 *  value would be used as a NTP server respectively indirectly adjusted by
 *  the NTP client function.
 *
 *  \param secTime308UTC the time in seconds since March 2008 UTC
 *  \sa  secClock()
 *  \sa secTime308UTC()
 */
uint32_t secTimeNtpUTC(uint32_t secTime308UTC) __attribute__((always_inline));

/** The reset / start time in seconds since March 1st 2008 UTC
 *
 *  This is the systems reset time in seconds in seconds since
 *  \ref intro_sec "weAutSys"'s zero date March 1st 2008 UTC. It will be
 *  initialised by \ref DEFAULT_START_TIME and be adjusted by the first
 *  setting of time and date (e.g. by NTP client function) after reset.
 */
#define startTime308UTC secDurTimers.period.v32


/** The combined offset (local - UTC)
 *
 *  This is the difference from  \ref secTime308Loc "local time" to
 *  \ref secTime308UTC "UTC".<br />
 *  In other words it is the sum \ref zoneOffsetSec + \ref setDST "DST offset"
 *
 *  Do never ever modify this value directly. That may spoil timers and more.
 */
extern uint32_t combinedOffset;

/** Is current time DST
 *
 *  This is non 0 if (current) \ref secTime308Loc() is summertime.
 *  \sa setDST()
 */
extern uint8_t isDST;

/** Set if current time is  DST
 *
 * \return 0 if no changes were made, otherwise 1
 * \sa isDST
 */
uint8_t setDST(uint8_t dlt);

//-------------------- UTC / zone -------------------------------------

/** The time zone offset in seconds
 *
 *  This is usually in multiples of hours (3600) and in the range of
 *  -12..+12 (*3600) &mdash; positive being east of Greenwich.
 *
 *  The value should usually set by DHCP, the reset / default is 3600, i.e.
 *  CET.
 */
#define zoneOffsetSec msTimers.period.v32
#define zoneOffsetZone msTimers.period.v8[1] //!< zoneOffsetSec's byte [1]

/** Adjust / set zone offset
 *
 *  This function sets \ref zoneOffsetSec and handles all side effects in the
 *  case of a change.
 *
 *  \return true if changed
 */
uint8_t setZoneOffsetSec(uint32_t newOffset);


/** Reset (or default) start time
 *
 *  This absolute time in seconds since March 1st 2008 that will usually be
 *  set as last compile time (in the file maketime_utc.h
 *  by utc_maketime.bat). <br />
 *
 *  This time will be used as reset value for the internal seconds
 *  clock. <br />
 *  Of course it will be quite wrong (i.e. more or less in past) until any
 *  adjustments by human command (CLI), time server answers or else are
 *  available.
 *
 *  \c DEFAULT_START_TIME can be used as date and time of make but will
 *  usually differ from \ref SYST_DAT, which is the "official" (sub)
 *  version date.
 */
#define DEFAULT_START_TIME (MAKETIME_UTC - OFFSET2UNIXTIME)


/** March 2008 as NTP seconds
 *
 *  This is the NTP date for March 1 2008 00:00 UTC.<br />
 *  In \ref intro_sec "weAutSys"'s seconds based \ref secClock
 *  "absolute time" this is the zero date.
 *  \showinitializer
 */
#define MARCH_2008_NTP 3413318400
#define OFFSET2NTPTIME 3413318400 //!< same as \ref MARCH_2008_NTP

/** March 2008 as UNIX seconds
 *
 *  This is the UNIX date for March 1 2008 00:00 UTC
 *  (\ref intro_sec "weAutSys"'s zero date).
 *  \showinitializer
 *  \sa  MARCH_2008_NTP
 */
#define MARCH_2008_UNIX 1204329600
#define OFFSET2UNIXTIME 1204329600 //!< same as \ref MARCH_2008_UNIX

/** @name Date calculation defines
 */
///@{

/** Days in a year
 *
 *  It is of course 365; a leap year will be 1 day longer.
 *
 *  This leap day, the 29th of February (29.2.leapYear), would better
 *  have been put at the end of the year by the popes. That is in effect
 *  done so by clever calendar algorithms (not invented here but heavily
 *  used in  \ref intro_sec "weAutSys").
 *  \hideinitializer
 */
#define DAYS_IN_YEAR 365

/** Days in four years
 *
 *  It is of course 4 * 365 + 1.
 *  \hideinitializer
 */
#define DAYS_IN4YEARS 1461

/** Offset of April
 *
 *  Days from March 1st to April 1st (same year).
 *  \hideinitializer
 */
#define APRIL_OFF 31  // length of March

/** Offset of May
 *
 *  Days from March 1st to May 1st (same year).
 *  \hideinitializer
 */
#define MAY_OFF 61  // length of March + April

/** Offset of June
 *
 *  Days from March 1st to June 1st (same year).
 *  \hideinitializer
 */
#define JUNE_OFF 92  // length of March + April +

/** Offset of July
 *
 *  \hideinitializer
 *  \sa JUNE_OFF
 */
#define JULY_OFF 122

/** Offset of August
 *
 *  \hideinitializer
 *  \sa JUNE_OFF
 */
#define AUGUST_OFF 153

/** Offset of September
 *
 *  \sa JUNE_OFF
 *  \hideinitializer
 */
#define SEPTEMBER_OFF 184

/** Offset of October
 *
 *  \sa JUNE_OFF
 *  \hideinitializer
 */
#define OCTOBER_OFF 214

/** Offset of November
 *
 *  \sa JUNE_OFF
 *  \hideinitializer
 */
#define NOVEMBER_OFF 245

/** Offset of December
 *
 *  Days from March 1st to December 1st (same year).
 *  \sa JUNE_OFF
 *  \hideinitializer
 */
#define DECEMBER_OFF 275

/** Offset of (next) January
 *
 *  Days from March 1st to January 1st (next year).
 *  \sa DECEMBER_OFF
 *  \hideinitializer
 */
#define JANUARY_OFF 306

/** Offset of (next) February
 *
 *  Days from March 1st to February 1st (next year).
 *  \hideinitializer
 */
#define FEBRUARY_OFF 337

//---------------------------------------   certain dates as days  ----

/** March 2012
 *
 *  This is the number of days at march 1st 2012 (1.3.2012 / 2012-03-01)
 *  counted since March 1st 2008 (1.3.2008 / 2008-03-01).
 *  \hideinitializer
 */
#define MARCH_2012 1461   // 1 *  DAYS_IN4YEARS

/** March 2016
 *
 *  This is the number of days at march 1st 2016 (1.3.2016 / 2016-03-01)
 *  counted since March 1st 2008 (1.3.2008 / 2008-03-01).
 *  \hideinitializer
 */
#define MARCH_2016 2922   // 2 *  DAYS_IN4YEARS

/** March 2020
 *
 *  This is the number of days at march 1st 2020 (1.3.2020 / 2020-03-01)
 *  counted since March 1st 2008 (1.3.2008 / 2008-03-01).
 *  \hideinitializer
 */
#define MARCH_2020 4383   // 3 *  DAYS_IN4YEARS

/** March 2100
 *
 *  This is the number of days at march 1st 2100 (1.3.2100 / 2100-03-01)
 *  counted since March 1st 2008 (1.3.2008 / 2008-03-01).
 *  \hideinitializer
 */
#define MARCH_2100 33602   // 23 * DAYS_IN4YEARS - 1

///@} // end member group  Date calculation
/**
 * @}
 * \addtogroup timerHdl
 * @{
 */


//--------   Timers    ------------------------------------------------

#define TIMER_STATE_MASK 0xF0 //!< upper 4 bits of timer.state (the state)
#define TIMER_TYPE_MASK 0x03  //!< lower 4 (2) bits of timer.state (the type)

/** A milliseconds based timer
 *
 *  This is a timer with milliseconds resolution that is duration / period
 *  / frequency oriented. Its endTime relates to \ref msClock.
 */
#define ms_TIMER_TYPE 0

/** A duration / period oriented timer with seconds resolution
 *
 *  This is a timer with seconds resolution that is duration / period
 *  oriented. Its endTime relates to \ref secClock.
 */
#define sec_dur_TIMER_TYPE 2

/** A date / time oriented timer with seconds resolution
 *
 *  This is a timer with seconds resolution that is date / time oriented.
 *
 *  Its endTime relates to \ref secClock. In effect the endtimes of
 *  \ref TIMER_RUNNING "running" timers of that type are adjusted by the
 *  the same amount if any date and time settings or adjustments should
 *  occur. <br />
 *  So, for example, a timer of this type started with \ref SECONDS_IN_WEEK
 *  duration an Monday 7:15, will lapse next Monday 7:15 no matter what
 *  adjustments (by NTP client e.g.) or even \ref isDST "summertime" shifts
 *  were made during that week. On the other hand big adjustments (like
 *  setting date and time to totally different values) may bring a timer
 *  of this type far into future or let it lapse at once.
 */
#define sec_dat_TIMER_TYPE 1


/** Timer free
 *
 *  This is the type of a timer that is neither used nor owned by any task.
 *  If used as type value for (re-) start, this acts as
 *  \ref TIMER_TYPE_DEFAULT "default".
 *
 *  If the user / application software chooses to have a pool of timers (en
 *  lieu de one static timer for every purpose) it will have to handle the
 *  transitions to / from this state.
 *
 *  \sa TIMER_LAPSED
 *  \sa TIMER_STATE_MASK
 *  \sa initTimer()
 *  \hideinitializer
 */
#define TIMER_TYPE_FREE 3
#define TIMER_TYPE_DEFAULT 3 //!< Set type to default or leave untouched

/** Timer lapsed
 *
 *  This is the state value of a timer that has run to the end of its
 *  (actual) period, i.e. lapsed or expired.
 *
 *  \sa TIMER_RUNNING
 *  \sa TIMER_STOPPED
 *  \sa TIMER_UNUSED
 *  \sa TIMER_STATE_MASK
 *  \showinitializer
 */
#define TIMER_LAPSED 0x00

/** Timer running
 *
 *  This is the state value of a timer that is running and not yet lapsed
 *  nor stopped.
 *
 *  \note This state is equivalent of being held in a system's list of
 *  running timers and must never be modified directly.
 *
 *  \sa TIMER_LAPSED
 *  \sa TIMER_TYPE_FREE
 *  \sa TIMER_STATE_MASK
 *  \hideinitializer
 */
#define TIMER_RUNNING 0x10

/** Timer stopped
 *
 *  This is the state value of a timer that has been stopped. Its period
 *  value will be set to the remaining rest time; it will have to be restored
 *  to its original value before a timer restart.
 *
 *  This usually makes no sense for \ref sec_dat_TIMER_TYPE "date time"
 *  oriented timers
 *
 *  \sa TIMER_LAPSED
 *  \sa TIMER_STATE_MASK
 *  \hideinitializer
 */
#define TIMER_STOPPED 0x20

/** Timer unused
 *
 *  This is the state value of a timer that is currently unused but still
 *  owned by a application or system task. This owner is responsible for
 *  re-using or freeing that timer.
 *
 *  \sa TIMER_LAPSED
 *  \sa TIMER_STATE_MASK
 *  \hideinitializer
 */
#define TIMER_UNUSED 0x40


/** The timer data type resp. structure
 *
 *  This type defines \ref intro_sec "weAutSys"' timers of 32 bit resolution in ms or s.
 *
 *  \note All fields of a timer may be read by application software even
 *  if that makes only partial sense. But !<br />
 *  No field of a timer must directly be set by application software.<br />
 *  It's best to use only the timer functions for manipulating timers.
 *
 *  \sa msClock(void)
 *  \sa s_time(void)
 *  \sa TIMER_LAPSED
 */
struct timer_t {

/** The timer's end (lapse) time
 *
 *  For a ms resolution timer:<br />
 *  This is the end time of a running timer in the (absolute) scale of
 *  ref msClock "ms since" system reset / restart (\ref msClock).<br />
 *  End time may be 28,6 days in the future for this timer type.
*
 *  For a seconds resolution timer:<br />
 *  This is the end time of a running timer in the (absolute) scale of
 *  ref secClock "seconds since" system reset / restart (\ref secClock)
 *  respectively \ref  secTime308Loc() "seconds since March 2008"
 *  local time (\ref  secTime308Loc()).<br />
 *  End time may be 68 years in the future for this timer type.
  *
 *  \sa TIMER_RUNNING
 */
   ucnt32_t end;

/** The timer's period
 *
 *  This is the actual period for the timer's current run or next re-start.
 *  \sa msClock(void)
 *  \sa secClock
 *  \sa TIMER_RUNNING
 */
   ucnt32_t period;

/** The timer's state and type
 *
 *  \sa TIMER_LAPSED
 *  \sa TIMER_RUNNING
 *  \sa TIMER_STOPPED
 *  \sa TIMER_TYPE_FREE
 *  \sa TIMER_UNUSED
 *  \sa ms_TIMER_TYPE
 *  \sa sec_dur_TIMER_TYPE
 *  \sa sec_dat_TIMER_TYPE
 */
   uint8_t state;

/** The next timer
 *
 *  This points to the next timer if this timer is in any system maintained
 *  list. <ul>
 *  <li> This is for system software only! </li></ul>
 *  The value must not be modified by user software;
 *  see also  \ref do_not_mod_warn "warnings above".
 *
 *  The only exception is for user / application software that chooses to
 *  handle a pool of timers and if the state of a pool timer is
 *  \ref TIMER_TYPE_FREE.
 */
   struct timer_t * next;
}; // timer_t


/** The system milliseconds time and timers
 *
 *  This is (a pseudo timer) for system internal timing and list handling
 *  use only!
 *
 * \c .next is the root of the system's internal ordered linked
 *  list of active / running ms based timers.
 *
 * \c .end is the (one) system ms 32 bit counter.
 *
 * \c .period is the time zone offset in seconds (0 for WET, 3600 for CET,
 *  7200 for CEST [EU DST rules implied]; -5 for EST .. -8 PST [Northamerican
 *  DST rules]).
 *
 *  Application software must not modify any field.
 */
   extern struct timer_t msTimers;

/** The system seconds resolution date / time oriented timers
 *
 *  This is (a pseudo timer) for system internal timing  and list handling
 *  use only!
 *
 *  \c .next is the root of the system's internal ordered linked
 *  list of active / running seconds based timers.
 *
 *  \c  secDatTimers.end  is the local time in seconds since
 *  March 1st 2008 UTC.
 *
 *  \c  secDatTimers.period is the (\c msTimers.end stamp) value of the
 *  absolute date / time last adjustment or setting.
 *
 *  Application software must not modify any field.
 */
   extern struct timer_t secDatTimers;

/** The system seconds resolution duration oriented timers
 *
 *  This is (a pseudo timer) for system internal timing  and list handling
 *  use only!
 *
 *  \c .next is the root of the system's internal ordered linked
 *  list of active / running seconds based timers.
 *
 *  \c  secDurTimers.end is the (one) system seconds clock counting the
 *  seconds since last reset / re-start.
 *
 *  \c  secDurTimers.period is the system start time in
 *  seconds since March 1st 2008 UTC.
 *
 *  Application software must not modify any field.
 */
   extern struct timer_t secDurTimers;

/** The list of timer lists
 *
 *  Index is the type: \ref ms_TIMER_TYPE \ref sec_dat_TIMER_TYPE
 *  \ref sec_dur_TIMER_TYPE ([0..2])
 *
 *  \sa TIMER_TYPE_MASK
 */
  extern struct timer_t * timerLists[];

/** Remove a timer from a list
 *
 *  This function removes the \c timer from the \c list and then sets
 *  \c timer->next to NULL. The return value will point to the element that
 *  was \c timer's predecessor in \c list.
 *
 *  If \c timer is not found in \c list NULL is returned and neither \c timer
 *  nor \c list will be touched.
 *
 *  \note No parameter must be NULL !
 *  \note This is a (system) helper function only. No checks are made if the
 *  call is legal or makes sense.
 *
 * \param timer the timer to be removed from list
 * \param list the list head, but may be any timer (in a linked list) the
 *        search for timer to be removed will start after
 */
struct timer_t * removeTimerFromList(const struct timer_t * timer,
                                                       struct timer_t * list);

/** Start a timer with interval / type  from now
 *
 *  If timer is NULL or it is \ref TIMER_TYPE_FREE "free" nothing will
 *  happen. <br />
 *  If the \c period is set to 0 the state will be set to
 *  \ref TIMER_UNUSED "unused" (n.b. not to \ref TIMER_LAPSED "lapsed") and
 *  nothing else will happen.
 *
 *  Otherwise this function will set the \c timer with \c period ms or s
 *  \ref TIMER_RUNNING "running" from now, no matter what its before state
 *  has been.
 *
 *  \param timer points to the timer to be started
 *  \param period is the next interval duration in s or ms depending on \c type
 *  \param type may be \ref ms_TIMER_TYPE,  \ref  sec_dur_TIMER_TYPE or
 *              \ref  sec_dat_TIMER_TYPE; the
 *              \ref  TIMER_TYPE_MASK "upper 6 state" bits are ignored
 */
void startTimer(struct timer_t * timer, uint32_t period, uint8_t type);

/** Start a timer with its own interval from now
 *
 *  If timer is NULL or it is \ref TIMER_TYPE_FREE "free" nothing will
 *  happen. <br />
 *  If its \c period is 0 the state will be set to
 *  \ref TIMER_UNUSED "unused" and nothing else will happen.
 *
 *  Otherwise this function will set the \c timer with its own \c period
 *  \ref TIMER_RUNNING "running" from now, no matter what its before state
 *  has been.
 *
 *  \param timer points to the timer to be (re-) started
 */
void restartTimer(struct timer_t * timer);

/** Start a timer (with absolute end time)
 *
 *  If timer is NULL or it is \ref TIMER_TYPE_FREE "free" nothing will
 *  happen.<br />
 *  If the \c endTime set is not in the future the state will be set to
 *  \ref TIMER_UNUSED "unused" (n.b. not to \ref TIMER_LAPSED "lapsed") and
 *  nothing else will happen.
 *
 *  Otherwise this function will set the \c timer
 *  \ref TIMER_RUNNING "running" to the \c endTime (of \c type ).
 *
 *  \param timer points to the timer to be started
 *  \param endTime is the next interval's end in s or ms depending on \c type
 *  \param type may be \ref ms_TIMER_TYPE,  \ref  sec_dur_TIMER_TYPE or
 *              \ref  sec_dat_TIMER_TYPE; the
 *              \ref  TIMER_TYPE_MASK "upper 6 state" bits are ignored
 */
void startTimerAbs(struct timer_t * timer, uint32_t endTime, uint8_t type);


/** Start a timer's next interval
 *
 *  If timer is NULL or it is \ref TIMER_TYPE_FREE "free" nothing will
 *  happen.<br />
 *  If the \c period is 0 nothing  will happen, also.<br />

 *  Otherwise this function starts or restarts the \c timer with its actually
 *  set internal \c period from its actual end time.<br />
 *  This actual end time may be in future for a running timer. In this case
 *  the timer's next lapse will be postponed farther in the future.<br />
 *  The actual end time may be now or in the past for a lapsed timer.
 *  In this case it is a "frequency exact" restart, while
 *  \ref startTimer() would be used for an "absolute time exact" restart.
 *
 *  Attention: If the the timers actual end time so far in the past that the
 *  prolonged end time is in the past too, the timer will not be re-started.
 *  It will be lapsed at once.
 *
 *  If the timer is in state halted/paused or unused it will start with its
 *  internal settings from now.
 *
 *  \param timer points to the timer to be re-started
 */
void startNextInterval(struct timer_t * timer);

/** Halt a timer
 *
 *  If timer is NULL or it is \ref TIMER_TYPE_FREE "free" nothing will
 *  happen.<br />
 *
 *  Just stop the timer. This will set the timer to state \c TIMER_UNUSED.
 *
 *  \param timer points to the timer to be halted
 */
void haltTimer(struct timer_t * timer);

/** Initialise a timer
 *
 *  This will set the timer to state \ref TIMER_UNUSED "unused" and to the
 *  given  \c period and \c type non-regarding its current state.
 *
 *  This function should also be called by the owner of a (static!) timer
 *  for first time (reset) initialisation. If the user / application software
 *  chooses to have a pool of N timers (probably as static timer_t tPool[N])
 *  this function should be called for every element.
 *
 *  If \c timer is NULL nothing will be done; if its state is
 *  \ref TIMER_RUNNING it will be removed from the system's list of
 *  running timers.
 *
 *  \sa freeTimer
 *  \param timer points to the timer to be initialised
 *  \param period the duration in s or ms dependinf on \c type
 *  \param type may be \ref ms_TIMER_TYPE,  \ref  sec_dur_TIMER_TYPE or
 *              \ref  sec_dat_TIMER_TYPE; the
 *              \ref  TIMER_STATE_MASK "upper 4 state" bits are ignored
 *  \return false (0) if the previous state of \c timer was
 *          not \ref TIMER_TYPE_FREE "free", otherwise, as a signal to user
 *          software that chose to handle (a list of) free timers, true (!=0)
 */
uint8_t initTimer(struct timer_t * timer, uint32_t period, uint8_t type);

/** Stop / pause a timer
 *
 *  If timer is NULL, \ref TIMER_UNUSED "unused"  or
 *  \ref TIMER_TYPE_FREE "free" nothing will happen.<br />
 *
 *  A bit like \c haltTimer(struct timer_t), but the internal \c period will
 *  be reduced to the actual run duration not yet elapsed and the state will
 *  be \ref TIMER_STOPPED.
 *
 *  \param timer pointer to the timer to be freed
 */
void pauseTimer(struct timer_t * timer);

/** Free a timer
 *
 *  This function sets a timer in the type \ref TIMER_TYPE_FREE.
 *
 *  If \c timer is NULL nothing will be done; if it is
 *  \ref TIMER_RUNNING "running" it will be removed from the system's
 *  list of running timers.
 *
 *  \sa initTimer
 *  \param timer pointer to the timer to be freed
 */
void freeTimer(struct timer_t * timer);


/** Is a timer lapsed
 *
 *  This will evaluate to true if the timer was started and has run
 *  to its end.
 *  \param  timer The timer to be asked (never NULL !)
 *  \hideinitializer
 *  \sa timerRunning
 */
#define timerLapsed(timer) (!((timer).state & TIMER_STATE_MASK))

/** Is a timer running
 *
 *  This will evaluate to true if the timer is still running; i.e. has not
 *  yet run to its end.
 *
 *  \note timerLapsed() and !timerRunning are not the same.
 *
 *  \param  timer The timer to be asked (never NULL !)
 *  \hideinitializer
 *  \sa timerLapsed
 */
#define timerRunning(timer) (((timer).state & TIMER_STATE_MASK) == TIMER_RUNNING)


//----------------------------------------------------------------------------
/**
 * @}
 * \addtogroup timeKeep
 * @{
 */


/** "Time since" as a structure: a duration or a date/time
 *
 *  This type stores the elapsed time as "seconds, .., days" relative
 *  to a arbitrary starting zero. It is the "structured" equivalent to a
 *  32 bit unsigned "seconds since" value.
 *
 *  Depending on (the semantic of) this zero point in time this always
 *  represents  a duration  but in case of an appropriate "zero" also
 *  an absolute date and time.
 *
 *  Due to said equivalence an unsigned 32 bit seconds value can be
 *  \ref sec2datdur "converted" to this type and
 *  \ref datdur2sec "vice versa". So it is commonly used as intermediate form
 *  for parsing and \ref formTimDur "formatting".
 */
typedef struct {
/** The seconds in a minute
 *
 *  This value will wrap from 59 to 0 (after one minute).
 */
   uint8_t s;

/** The minute in an hour
 *
 *  This value wrap from 59 to 0 (after one hour).
 */
   uint8_t m;

/** The hour in a day
 *
 *  This value wrap from 23 to 0 (after one day).
 */
   uint8_t h;

/** The day
 *
 *  This day counter will wrap after about 179 years.<br />
 *  If used as offset to 1st of March 2008 (1.3.2008 / 2008-03-01) this
 *  absolute date will get us to 2187 and hence far beyond 2100
 *  respectively 2076 (2008 + 136/2), see also \ref getDaysByDat().
 */
   uint16_t d;
} datdur_t;

/** Convert seconds to a datdur_t structure
 *
 *  This function converts a 32 bit unsigned seconds value to a time
 *  structure of type \ref datdur_t. Its interpretation as duration or
 *  absolute date and time depends on the parameter's semantic.
 *
 *  Hint: This is a quite expensive operation for an 8 bit RISC machine
 *  without divide instruction. The result \c timStr should be kept if
 *  usable more than once.
 *
 *  \param  timStr  pointer to the time structure to be set
 *  \param  timSec  the time in seconds
 *  \sa datdur2sec
 */
void sec2datdur(datdur_t * timStr, uint32_t timSec);

/** Calculate seconds from  a datdur_t structure
 *
 *  This function converts the content of a structure of type \ref datdur_t
 *  to the corresponding 32 bit unsigned seconds value. It is the inverse of
 *  \ref sec2datdur().
 *
 *  Hint: This function is not quite cheap for an 8 bit RISC machine with
 *  just a 8bit * 8bit = 16bit multiply instruction. The returned result
 *  should be kept if needed more than once.
 *
 *  \param  timStr  pointer to the time structure to be used
 *  \return the seconds calculated from timStr
 */
uint32_t datdur2sec(datdur_t * timStr);




/** Date structure: a date in our time
 *
 *  This type represents a date in the range 2000-01-01 to 2255-12-31
 *  including day-of-week. Regarding the supplied functions for converting,
 *  formatting and parsing and its internal use it is strongly recommended
 *  to restrict its use to \ref intro_sec "weAutSys"'s (currently) usable
 *  date range: 2008-3-1 .. +68 years.
 *
 *  In that restricted sense this is the "structured" equivalent to the
 *  absolute date as days since March 1st 2008
 *  (\ref intro_sec "weAutSys"'s day 0) &mdash; especially to the field d
 *  of a \ref datdur_t structure related to that "zero".
 *
 *  Due to this equivalence an unsigned 16 bit "number of days since" value
 *  can be \ref setDatByDays "converted" to this type and
 *  \ref getDaysByDat "vice versa".
 *
 *  This type it is also commonly used as intermediate form
 *  for parsing and formatting.
 */
typedef struct {

/** The day in the week
 *
 *  The range is 1..7 Sunday to Saturday. <br />
 *  The value 0 means unknown respectively not yet set consistent to the
 *  other fields.
 */
   uint8_t wd;

/** The year
 *
 *  The (recommended / usable) range is 8..141 to be interpreted as
 *  2008 .. 2141
 */
   uint8_t y;

/** The month
 *
 *  The range is, of course, 1..12 as January..December.
 */
   uint8_t m;

/** The day in the month
 *
 *  The range is 1..28 | 29 | 30 | 31.
 */
   uint8_t d;

} date_t;

/** @name Date handling functions
 */
///@{

/** Set date structure by days since since March 2008
 *
 *  This function sets the (16 bit unsigned) number of days since March 2008
 *  into the structure datStr and all its other fields.
 *
 *  \param  datStr  pointer to the date structure to be set
 *  \param  ds      days since \ref intro_sec "weAutSys"'s day 0
 *  \return the resulting weekday (1..7) or 0 in erroneous situations
 *  \sa datdur2sec
 *  \sa getDaysByDat
 */
uint8_t setDatByDays(date_t * datStr, uint16_t ds);

/** Get days since since March 2008 by date structure
 *
 *  This calculates the (16 bit unsigned) number of days since March 2008
 *  from the structure and returns the result. All fields of datStr except
 *  day of week \c (ed) are used and must have correct and consistent values.
 *
 *  Before March 2008  0 is returned as is for everything after about ~2177.
 *
 *  \param  datStr  pointer to the date structure to be used
 *  \return     days since \ref intro_sec "weAutSys"'s day 0
 *  \sa setDatByDays
 *  \sa datdur2sec
 */
uint16_t getDaysByDat(date_t * datStr);

/**  Get year starting March (includes next January and February)
 *
 *  \param  ds      days since \ref intro_sec "weAutSys"'s day 0
 *  \param  daysInYear pointer to extra result: the the days
 *          in the respective year (0 = March 1st); if NULL this information
 *          is lost
 *  \return the resulting year 8 .. = 2008 ..
 *  \sa datdur2sec
 */
uint8_t getMarchYearByDays( uint16_t * daysInYear, uint16_t ds);

///@}

/** Rule structure: DST rules for a given (set of) year(s)
 *
 *  This type represents the daylight-saving time rules by a set of values
 *  needed by the the DST handling functions. The rules represented by a
 *  structure of this type support the EU and US DST rules.
 *
 *  The current (as of 2012) EU and US rules are made
 *  \ref getDSTrule "available" for the years
 *  2000 .. 2255. This will be longer than \ref intro_sec "weAutSys" may be
 *  used and especially longer than politicians will stick to the current
 *  EU and US rules.
 *
 *  Note: The statements (last "Sunday in ..") of the rules supplied will, of
 *  course, hold &mdash; but the DST laws will probably change.
 *
 *  As all dates and offsets for US and EU rules given here are after
 *  March 1st or are relative to this date the values will not be affected by
 *  leap day (February 29th). Hence (non regarding the infernal leap seconds)
 *  there are just 7 different sets of rules resp. rule structures numbered
 *  0 .. 6. This rule number may be obtained by \ref lastSunMarch - 25
 *  i.e. (first byte of structure) - 25.
 *
 *  \sa getDSTrule()
 */
typedef struct {

/** Day in month of last Sunday in March
 *
 *  For the set of years for which this rule is valid this will give the
 *  last Sunday in March. This is when EU switches DST on at 01:00 UTC, by
 *  turning the local times 1 hour ahead.
 *
 *  range: 25 .. 31
 */
   uint8_t lastSunMarch;

/** Day in month of last Sunday in October
 *
 *  For the set of years for which this rule is valid this will give the
 *  last Sunday in October. This is when EU switches DST off at 01:00 UTC, by
 *  turning the local times 1 hour back to normal. This will repeat the local
 *  time 02:00,000 .. 02:59,999 in CET, for example.
 *
 *  range: 25 .. 31
 */
   uint8_t lastSunOctober;

/** Number of seconds from March 1st to last Sunday in March 01:00
 *
 *  This is the number of seconds from March 1st 00:00 to
 *  last Sunday in March 01:00. This value supports the exact timing of
 *  EU switching DST on.
 */
   uint32_t lastSunMarch0100offset;

/** Number of seconds from March 1st to last Sunday in October 01:00
 *
 *  This is the number of seconds from March 1st 00:00 to
 *  last Sunday in October 01:00. This value supports the exact timing of
 *  EU switching DST off.
 */
   uint32_t lastSunOctober0100offset;

/** Day in month of second Sunday in March
 *
 *  For the set of years for which this rule is valid this will give the
 *  second Sunday in March. This is when most of the United States of America,
 *  Canada and Mexico  switch DST on at 02:00 (AM) local timeTC, by
 *  turning the local times 1 hour ahead.
 *
 *  range: 8 .. 14
 */
   uint8_t secondSunMarch;

/** Day in month of first Sunday in November
 *
 *  For the set of years for which this rule is valid this will give the
 *  first Sunday in November. This is when most Northern America witches DST
 *  off at 02:00 (AM).
 *
 *  range: 1..7
 */
   uint8_t firstSunNovember;

/** Number of seconds from March 1st to second Sunday in March 02:00
 *
 *  This is the number of seconds from March 1st 00:00 to
 *  second Sunday in March 02:00. This value supports the exact timing of
 *  US etc. switching DST on.
 */
   uint32_t secondSunMarch0200offset;

/** Number of seconds from March 1st to first Sunday in November 02:00
 *
 *  This is the number of seconds from March 1st 00:00 to
 *  first Sunday in November 02:00. This value supports the exact timing of
 *  US etc.  switching DST off.
 */
   uint32_t firstSunNovember0200offset;

} dst_rule_year_t;

/** Get the EU, US &amp;c DST rule data for a given year
 *
 *  This function returns a pointer to the
 *  \ref  dst_rule_year_t "DST rule structure" for the year requested in
 *  flash memory (!). Use pgm_read_byte() respectively pgm_read_dword()
 *  accordingly to get single elements. Or &mdash; to have the whole struture
 *  in RAM use \code
    dst_rule_year_t currentRuleValues;
    memcpy_P(&currentRuleValues, getDSTrule(year), sizeof(dst_rule_year_t));
 *  \endcode
 *  \param year 0..255 interpreted as 2000..2255
 */
const dst_rule_year_t  *  getDSTrule(uint8_t year);

/** Is date represented in date structure EU DST switching day
 *
 *  This function checks if the date is either the last Sunday in March
 *  (returns 3) or the last Sunday in October (returns 10) or non of
 *  both (returns 0).
 *
 *  \param  datStr  pointer to the date structure
 *  \return the resulting 3, 10 or 0
 */
uint8_t isEUdstSwitchDay(date_t * datStr);



/** @} */ // Doxygen addtogroup timeKeep
/** @} */ // Doxygen addtogroup sysSer

/**
 * \addtogroup  utilHelp
 * @{
 * \addtogroup  utilForm
 * @{
 */

/** Format a time (datdur_t structure) as duration
 *
 *  This function formats a time structure \ref datdur_t as duration.<br />
 *  The format will be \code "1234d 23h59m59s"\endcode. 16 characters will
 *  be put to \c *s including a trailing zero.
 *
 *  \param s     start position within string resp. char array (not NULL!)
 *  \param tRun  pointer to the time structure (not NULL!)
 *  \return points to the (0-) character behind the formated duration
 */
char * formTimDur(char * s, datdur_t * tRun);

/** Format a time (datdur_t structure) as time of day
 *
 *  This function formats a time structure \ref datdur_t as time of day or
 *  clock time. <br />
 *  The format will be \code "23:59:59"\endcode. 9 characters will
 *  be put to \c *s including a trailing zero.
 *
 *  The field \c d of \c tTim is not used for the result.
 *
 *  \param s     start position within string resp. char array
 *  \param tTim  pointer to the time structure
 *  \return points to the (0-) character behind the formated duration
 */
char * formTimOfD(char * s, datdur_t * tTim);

/** Format the day of week as short clear text
 *
 *  This function puts the three character short English day of week into
 *  \c s. <br />
 *  The format for \c wd 1..7 will be
 *  \code "Sun" "Mon" "Tue" "Wed" "Thu" "Fri" "Sat" \endcode. <br />
 *  4 characters will be put \c *s including a trailing zero.
 *
 *  For \c wd outside 1..7 just \c *s will be set 0 and \c s is returned.
 *
 *  \param s     start position within string resp. char array
 *  \param wd    1..7 is Sunday .. Saturday
 *  \return points to the (0-) character behind the formated day of week
 */
char * formWdShort(char * s, uint8_t wd);

/** Format a date (date_t structure)
 *
 *  This function formats a date (structure \ref date_t).<br />
 *  The format (ISO 8601 / EN 28601) will be \code "2009-01-31"\endcode.<br />
 *  11 characters will be put \c *s including a trailing zero.
 *
 *  The field \c wd is not used for the result.
 *
 *  \param s     start position within string resp. char array
 *  \param tDat  pointer to the time structure
 *  \return points to the (0-) character behind the formated day of week
 */
char * formDateIso(char * s, date_t * tDat);

extern char* wDaysShort[];  //!< flash array of the (flash) short weekdays


/** @} */ // Doxygen addtogroup utilForm
/**
 * \addtogroup  utilPars
 *  @{
 */

/** Parse a (clock) time
 *
 *  This function tries to interpret a part of the string \c s starting at
 *  \c si as a (clock) time. The expected format is "hh:mm:ss" or "hh:mm"
 *  with the seconds implied as 0. Returned is the index of the first
 *  character after that (parsed) time in case of success and 0 otherwise.
 *
 *  Leading and trailing blanks at every segment will be skipped as will be
 *  (allowed) leading zeroes. Hence "17:3:9" and " 17 : 03 :09 " will get
 *  the same interpretation.
 *
 *  \param tim pointer to the result structure; may be changed partially
 *             even if the parse was not successful
 *  \param s   the RAM string containing the time
 *  \param si  the first time's index in s
 *  \return    the index of the first character after the parsed time
 *             in case of success or 0 else
 */
uint8_t parseTim(datdur_t * tim, char s[], uint8_t si);

/** Parse a date
 *
 *  This function tries to interpret a part of the string \c s starting at
 *  \c si as a (clock) time. The expected format is "yyyy-mm-dd" or
 *  "yy-mm-dd" with two digit year interpreted as 2008 .. 2107 which is more
 *  than the recommended / allowed range of years.
 *
 *  Leading and trailing blanks at every segment will be skipped as will be
 *  (allowed) leading zeroes. Hence "12-18-1" and " 2012 - 18 - 01 " and
 *  even " 2012 - 18 - 0x00001 " will get the same interpretation.
 *
 *  This function does only rudimentary checks to the ranges in the date
 *  string ((20)08 .. 2169, 1..12, 1..31). If one check fails or the string
 *  can't be interpreted 0 is returned. Fields in \ dat may have been
 *  changed nevertheless. The field \c wd (week day) in \c dat
 *  is set to 0 (illegal / unknown) instead of the correct (1..7) value in
 *  case of failure.
 *
 *  \param dat pointer to the result structure; may be changed partially
 *             even if the parse was not successful
 *  \param s   the RAM string containing the date
 *  \param si  the first index of the date in s
 *  \return    the index of the first character after the parsed time
 *             in case of success or 0 else
 */
uint8_t parseDate(date_t * dat, char s[], uint8_t si);

/** @} */ // Doxygen addtogroup utilPars
/** @} */ // Doxygen defgroup utilHelp


#endif /* _TIMING_H_*/

/** @} */ // Doxygen addtogroup weAutSys
