// Copyright (c) 2011 Albrecht Weinert, Bochum
// $Id: utils.h 2 2017-01-25 16:49:03Z albrecht $

/**
 * \addtogroup weAutSys
 * @{
 *
 * \addtogroup utilHelp
 * @{
 *
 * \file utils.h
 *
 *  \ref intro_sec "weAutSys"  utility / library functions
 *              to be used also by application / user software
 *
 *  This file contains the definitions for \ref intro_sec "weAutSys"'
 *  (weAut_01) utility / library functions to be used also by application
 *  / user software.
 *  This is system software and must not be modified for user or
 *  application programs.
 *
 * This file is part of \ref intro_sec "weAutSys" &nbsp;
 *           <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 2 $
 * $Date: 2017-01-25 16:49:03 +0000 (Wed, 25 Jan 2017) $
 */
//  SVN last change by $Author: albrecht $

#ifndef UTILS_H_
#define UTILS_H_


#include <avr/pgmspace.h>
#include "we-aut_sys/common.h"


//---------------------------------------------- arithmetic divide  -----
/**
 * \addtogroup utilArith
 * @{
 *
 * @name Optimised Divide functions
 */
///@{

/** Divide an unsigned byte by the constant 10
 *
 *  ATmega -controllers don't have any divide in their (RISC) instruction
 *  set. Nevertheless getting quotient and remainder by 10 in 8 bit unsigned
 *  arithmetic is needed quite often (even in loops).
 *
 *  Hence no effort was spared to make this a good implementation for
 *  \c uint8_t dividends. This function is faster than direct usage of
 *  avr-gcc C compiler's / and % operators and also faster than using
 *  stdlib.h's div(...,10). The same is true for all the other divide
 *  functions coming with \ref intro_sec "weAutSys" and its serial
 *  bootloader.<br />
 *
 *  \return the quotient and the remainder.
 *  \param div the dividend (byte, unsigned, 0..255)
 *  \sa divWByVal10toByte(uint16_t)
 *  \sa divWByVal1000(uint16_t)
 *  \sa twoDigs(char*, uint8_t)
 */
u8div_t divByVal10(uint8_t div) __attribute__ ((pure));

/** Divide an unsigned word by by the constant 10 for a byte quotient
 *
 *  ATmega -controllers don't have any divide in their (RISC) instruction
 *  set. Getting quotient and remainder by 10 is, nevertheless, needed quite
 *  often &mdash; especially for decimal formatting.
 *
 *  In that use case the dividends (\c div) are often in the range 0..999.
 *  That may be achieved for larger numbers by \ref divWByVal1000. Hence this
 *  is a highly optimised implementation for those small 16 bit dividends
 *  (0..999). It will fail without warning if the \c div is 1286 or above!
 *
 *  It's the user's responsibility not to use this function outside its
 *  operation range. On the other hand the implementation is much faster than
 *  all equivalents expressed with avr-gcc or libc functions.
 *
 *  \return the quotient and the remainder.
 *  \param div the dividend (16 Bit, unsigned, 0..1286)
 *  \sa divByVal10(uint8_t)
 *  \sa divWByVal1000(uint16_t)
 */
u8div_t divWByVal10toByte(uint16_t div) __attribute__ ((pure));


/** Divide an unsigned word by by the constant 1000
 *
 *  ATmega -controllers don't have any divide in their (RISC) instruction
 *  set. Getting quotient and remainder by 1000 is sometimes needed &mdash;
 *  for example as divide and conquer for subsequent
 *  \ref divWByVal10toByte "divide by 10" on decimal formatting.
 *
 *  This implementation returns quotient and the remainder as 16 bit values
 *  (in an u16div_t), even if 8 bit would suffice for the quotient.
 *
 *  \return the quotient and the remainder.
 *  \param div the dividend (16 Bit, unsigned, 0..65535)
 *  \sa  divWByVal10toByte(uint16_t)
 *  \sa  threeDigs(char*, uint16_t)
 */
u16div_t divWByVal1000(uint16_t div) __attribute__ ((pure));


/** Modulo 7 of an unsigned 16 bit value
 *
 *  This is an efficient result = val % 7 without using any divide functions.
 *  \param val the unsigned number
 *  \return   the remainder by 7 for val (0..6)
 */
uint8_t mod16byVal7(uint16_t val) __attribute__ ((pure));

/** Modulo 7 of an unsigned 8 bit value
 *
 *  This is an efficient result = val % 7 without using any divide functions.
 *  \param val the unsigned number
 *  \return   the remainder by 7 for val (0..6)
 */
uint8_t mod8byVal7(uint8_t val) __attribute__ ((pure));

///@} // end member group Divide functions
/**
* @name Optimised Multiplication functions
*/
///@{


/** Multiply unsigned 16 bit with 8 bit
 *
 *  This function calculates the the product of an 8 and a 16 bit number
 *  and returns the (24 bit) result as 32 bit.
 *
 *  The (ASM) implementation is far more efficient and the usage is less
 *  error prone as what could be expressed in C and then made by (AVR-) GCC.
 *
 *  \param f16 the 16 bit (unsigned) factor
 *  \param f8  the 8 bit (unsigned) factor
 *  \return the product f16 * f8 (It is a 24 bit value, bits 24..31 are 0)
 *  \sa  mul16
 *  \sa  mul16with17
 *
 */
uint32_t mul16with8(const uint16_t f16, const uint8_t f8) __attribute__ ((pure));

/** Multiply unsigned 16 bit with 16 bit
 *
 *  This function calculates the the product of two 16 bit numbers
 *  and returns the 32 bit result.
 *
 *  The (ASM) implementation is far more efficient and the usage is less
 *  error prone as what could be expressed in C and then made by (AVR-) GCC.
 *
 *  \param ab the first 16 bit (unsigned) factor
 *  \param cd the second 16 bit (unsigned) factor
 *  \return the product ab * cd
 *  \sa  mul16with8
 *  \sa  mul16with17
 */
uint32_t mul16(const uint16_t ab, const uint16_t cd) __attribute__ ((pure));

/** Multiply unsigned 16 bit with 17 bit
 *
 *  This function calculates the the product of a 16 bit number with a
 *  17 bit number and returns the 32 bit result. The second factor \c cd is
 *  supplied as 16 bit unsigned number with bit 16 implied as 1. Hence the
 *  second factor is in the range 65536 .. 131071, as is e.g
 *  \ref SECONDS_IN_DAY.
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param ab the first 16 bit (unsigned) factor
 *  \param cd  the second 17 bit (unsigned) factor
 *  \return the product ab * (cd + 0x10000)
 *  \sa  mul16with8
 *  \sa  mul16
 */
uint32_t mul16with17(const uint16_t ab, const uint16_t cd) __attribute__ ((pure));

/** Multiply unsigned 32 bit with the constant 512
 *
 *  This function calculates the the product of a 32 bit number with the
 *  constant 512 and returns the 32 bit result. This multiplication is needed
 *  for certain memory card operations (sector number to byte address e.g.).
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param fac the 32 bit (unsigned) factor <br />
 *             fac's upper 9 bits are ignored and should be 0 to get a
 *             arithmetically correct result
 *  \return the product fac * 512
 *  \sa mul16with8
 *  \sa mul16
 *  \sa div32byVal512
 */
uint32_t mul32withVal512(const uint32_t fac) __attribute__ ((pure));

/*  Multiply unsigned 16 bit with the constant 512
 *
 *  This function calculates the the product of a 16 bit number with the
 *  constant 512 and returns the 16 bit result. This multiplication is needed
 *  for certain memory card operations (sector number to byte address e.g.).
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param fac the 16 bit (unsigned) factor <br />
 *             fac's upper 9 bits are ignored and should be 0 to get a
 *             arithmetically correct result
 *  \return the product fac * 512
 */
uint16_t mul16withVal512(const uint16_t fac)  __attribute__ ((pure));

///@} // end member group Multiplication  functions
/**
* @name Optimised Divide functions
*/
///@{

/** Divide unsigned 32 bit by the constant 512
 *
 *  This function calculates the the quotient of a 32 bit number resulting
 *  from division by 512 and returns the 32 bit result. This division is
 *  needed for certain memory card operations (with 512 byte sectors e.g.).
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param div the 32 bit (unsigned) dividend
 *  \return the quotient div / 512
 *  \sa mul32withVal512
 *  \sa div16byVal512
 *  \sa div32byVal1024
 *  \sa div32byVal2048
 */
uint32_t div32byVal512(const uint32_t div) __attribute__ ((pure));

/** Divide unsigned 16 bit by the constant 512
 *
 *  This function calculates the the quotient of a 16 bit number resulting
 *  from division by 512 and returns the 16 bit result. This division is
 *  needed for certain memory card operations (with 512 byte sectors e.g.).
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param div the 16 bit (unsigned) dividend
 *  \return the quotient div / 512
 *  \sa div32byVal512
 *  \sa mul32withVal512
 *  \sa div32byVal1024
 *  \sa div32byVal2048
 */
uint16_t div16byVal512(const uint16_t div) __attribute__ ((pure));

/** Divide unsigned 32 bit by the constant 1024
 *
 *  This function calculates the the quotient of a 32 bit number resulting
 *  from division by 1024 and returns the 32 bit result. This division is
 *  needed for certain memory card operations.
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param div the 32 bit (unsigned) dividend
 *  \return the quotient div / 1024
 *  \sa  mul32withVal512
 */
uint32_t div32byVal1024(const uint32_t div) __attribute__ ((pure));

/** Divide unsigned 32 bit by the constant 2048
 *
 *  This function calculates the the quotient of a 32 bit number resulting
 *  from division by 2048 and returns the 32 bit result. This division is
 *  needed for certain memory card operations.
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param div the 32 bit (unsigned) dividend
 *  \return the quotient div / 2048
 *  \sa  mul32withVal512
 */
uint32_t div32byVal2048(const uint32_t div) __attribute__ ((pure));

/** Divide unsigned 32 bit by 256
 *
 *  This function calculates the the quotient of a 32 bit number resulting
 *  from division by 2048 and returns the 32 bit result. This division is
 *  needed for certain memory card operations.
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param div the 32 bit (unsigned) dividend
 *  \return the quotient div / 256
 *  \sa  mul32withVal512
 */
uint32_t div32byVal256(const uint32_t div) __attribute__ ((pure));

/** Divide unsigned 32 bit by the constant 128
 *
 *  This function calculates the the quotient of a 32 bit number resulting
 *  from division by 2048 and returns the 32 bit result. This division is
 *  needed for certain memory card operations.
 *
 *  This implementation is far more efficient as what could be expressed
 *  in C and then made by (AVR-) GCC.
 *
 *  \param div the 32 bit (unsigned) dividend
 *  \return the quotient div / 128
 *  \sa  mul32withVal512
 */
uint32_t div32byVal128(const uint32_t div)  __attribute__ ((pure));

/** Modulo by power of 2
 *
 *  This is a fast modulo implementation for an architecture without a
 *  divide instruction. As long as the (AVR-GC) C-compiler does not optimise
 *  power of 2 modulo this gives a quite significant performance improvement.
 *
 *  Warning: If \c po2 is not an unsigned power of 2 (i.e. just one bit set)
 *           this  will not give the correct arithmetic result.
 *
 *  \param div the (unsigned) dividend
 *  \param po2 the (unsigned) quotient (must be a power of 2 and >= 2)
 *  \return div % po2
 */
#define mod2pow(div, po2) (div & ((po2) -1))

///@} // end member group Divide functions
/**
* @name Optimised Compare functions
*/
///@{
//----------------------------------------------   arithmetic compare  -----

/** A 32 bit unsigned greater equal (ge) comparison for modulo arithmetic
 *
 *  This function will return true if a >= b, whereby a and b are considered
 *  as (counter) values in modulo 2**32 arithmetic. In that sense
 *  geU32ModAr(0x0001, 0xFFFE) will return true as  0x0001 is (as counter
 *  / modulo value) by 2 higher than 0xFFFE.
 *
 *  That will work correctly for differences less than 2**31 and, as said,
 *  very well over the "wrapping border".
 *
 *  \note Those who remember a Windows98 exploding after running a bit
 *  longer than 49 days will know why we need this type of comparison for
 *  a 32 bit milliseconds counter (e.g.).
 *
 *  \param a will be compared to
 *  \param b by a >= b in modulo arithmetic sense
 *  \sa msClock()
 *  \sa leU32ModAr
 */
uint8_t geU32ModAr(uint32_t a, uint32_t b)  __attribute__((always_inline));

/** A 32 bit unsigned less or equal (le) comparison for modulo arithmetic
 *
 *  This function will return true if a <= b, whereby a and b a considered
 *  as (counter) values in modulo 2**32 arithmetic. It is the counterpart
 *  to \ref geU32ModAr() (but, n.b., not the inverse).
 *
 *  \param a will be compared to
 *  \param b by a <= b in modulo arithmetic sense
 *  \sa geU32ModAr
 */
uint8_t leU32ModAr(uint32_t a, uint32_t b)  __attribute__((always_inline));

// the following are put to ll_common.h and implemented
// in weAutSys' serial bootloader
// uint16_t div16(uint16_t * rem, uint16_t dividend, uint16_t divisor);
// uint32_t div32(uint32_t * rem, uint32_t dividend, uint32_t divisor);


///@} // end member group Comparefunctions
/** @} */ // Doxygen addtogroup utilArith

//------------------------------------------------------    formatting -----
/**
 * \addtogroup utilForm
 * @{
 */

/** Set a three decimal digit number with leading zeroes into a string
 *
 *  This function converts a number 0..999 into three characters "000" ..
 *  "999". Values of 1000 and above will get "***".
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the value to convert
 *  \sa twoDigs(char*, uint8_t)
 *  \sa fourDigs(char*, uint8_t)
 *  \sa divWByVal1000(uint16_t)
 *  \sa divWByVal10toByte(uint16_t)
 *  \sa threeDigsB
 */
void threeDigs(char * begin, uint16_t value);

/**  Set a three decimal digit number with leading zeroes into a string
 *
 *  This function converts an eight bit  number 0..255 into three characters
 *  "000" .. "255".
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the value to convert
 *  \sa twoDigs(char*, uint8_t)
 *  \sa fourDigs(char*, uint8_t)
 *  \sa divWByVal1000(uint16_t)
 *  \sa divWByVal10toByte(uint16_t)
 *  \sa threeDigs
 */
void threeDigsB(char * begin, uint8_t value);

/** Set a four decimal digit number into a string
 *
 *  This function converts a number 0..9999 into three characters "   0" ..
 *  "9999". Values of 10000 and above will get " **0".
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the value to convert
 *  \sa twoDigs(char*, uint8_t)
 *  \sa threeDigs(char*, uint8_t)
 *  \sa fourHexs((char*, uint16_t)
 *  \sa divWByVal1000(uint16_t)
 *  \sa divWByVal10toByte(uint16_t)
 */
void fourDigs(char * begin, uint16_t value);


/** Set a two decimal digit number with leading zeroes into a string
 *
 *  This function converts a number 0..99 into two characters "00" ..
 *  "99". Values of 100 and above will get "**".
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the value to convert
 *  \sa threeDigs(char*, uint16_t)
 *  \sa  divByVal10(uint8_t)
 */
void twoDigs(char * begin, uint8_t value);

/** Set a two hexadecimal digit number with leading zeroes into a string
 *
 *  This function converts a number 0..255 into two characters "00" ..
 *  "FF".
 *
 *  \param s start position within the target string resp. char array
 *  \param value the value to convert
 *  \sa twoDigs(char*, uint8_t)
 *  \sa threeDigs(char*, uint16_t)
 *  \sa fourHexs((char*, uint16_t)
 *  \sa  divByVal10(uint8_t)
 */
void twoHexs(char * s, uint8_t value)  __attribute__((always_inline));

/** Set a four digit hexadecimal  number with leading zeroes into a string
 *
 *  This function converts a 16 bit number into four characters "0000" ..
 *  "FFFF".
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the value to convert
 *  \sa twoDigs(char*, uint8_t)
 *  \sa twoHexs(char*, uint8_t)
 *  \sa threeDigs(char*, uint16_t)
 *  \sa eightHexs(char *, uint32_t)
 *  \sa  divByVal10(uint8_t)
 */
void fourHexs(char * begin, uint16_t value);

/** Set an four digit big endian hexadecimal big endian number into a string
 *
 *  This function does the same as \ref eightHexs except for the wrong
 *  andianess of the \c value parameter.
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the big endian value to convert
 */
void fourHexsBE(char * begin, uint16_t value);

/** Set an eight digit hexadecimal number with leading zeroes into a string
 *
 *  This function converts a 32 bit number into eight  characters "00000000"
 *  .. "FFFFFFFF".
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the value to convert
 *  \sa twoDigs(char*, uint8_t)
 *  \sa twoHexs(char*, uint8_t)
 *  \sa threeDigs(char*, uint16_t)
 *  \sa fourHexs(char *, uint16_t)
 *  \sa eightHexsBE(char *, uint32_t)
 */
void eightHexs(char * begin, uint32_t value);

/** Set an eight digit big endian hexadecimal number with into a string
 *
 *  This function does the same as \ref eightHexs except for the wrong
 *  andianess of the \c value parameter.
 *
 *  \param begin start position within the target string resp. char array
 *  \param value the big endian value to convert
 */
void eightHexsBE(char * begin, uint32_t value);

/** Format a sequence of 16 bytes in four hex groups and as characters
 *
 *  This function formats the first 16 bytes pointed to by \c seq in four
 *  groups of 8 hexadecimal digits separated by space followed by said
 *  16 bytes as 16 character text. There control characters (0..0x20 and
 *  0x80..0xA0) are put as blank space.
 *
 *  The result's length is 55 characters including a trailing blank and
 *  terminating 0.<br />
 *  The result looks like <code>
 *  20746578 74206F6E 20612032 4720534D  text on a 2G SM </code>
 *
 *  \param s     start position within string respectively char array
 *  \param seq pointer to the 16 byte sequence to be formatted
 *  \return points to the (0-) character behind the formated result
 *  \sa formN20BytSeq
 */
char * form16BytSeq(char * s, uint8_t * seq);

/** Format a sequence of 0..20 bytes in five hex groups and as characters
 *
 *  This function formats the \c c (max. 20) bytes pointed to by \c seq in
 *  five groups of 8 hexadecimal digits separated by dot (.) followed by 20
 *  bytes as 20 character text. In this text field control characters
 *  (0..0x20 and 0x80..0xA0) as well as the bytes \c [>= \c n] are put as
 *  blank space.
 *
 *  The maximum length of the result (for \c n=20) is 68 characters
 *  including a trailing blank and terminating 0. <br />
 *  The result (for \c n=17)  looks like <code>
 *  20746578.74206F6E.20612032.4720534D.43....... text on a 2G SMC  </code>
 *
 *  \param s   start position within string resp. char array
 *  \param seq pointer to the up to 20 byte sequence to be formatted
 *  \param n   0..20 number of bytes
 *  \return points to the (0-) character behind the formated result
 *  \sa form16BytSeq
 */
char * formN20BytSeq(char * s, uint8_t * seq, uint8_t n);

/** Format a Modbus telegram
 *
 *  This function formats a Modbus TCP/IP telegram provided as structure
 *  \c modTeleg.. This consists of seven bytes called MBAB header plus 11..13
 *  bytes called PDU.
 *  From the MBAB the sequence number (#), bytes to follow (+) and the unit (u)
 *  are formatted. The PDU part starts with the function code (fx) followed by
 *  0..6 words (16 bit) The number and interpretation of the latter is
 *  dependent of the function code and direction (request / response).
 *
 *  The format returned is <br />
 *  " #.... +.... u.. fx.. .... .... .... .... .... .... .... " <br />
 *  All values are formated hexadecimal; 16 bit numbers are interpreted as
 *  (wrong) big endian.
 *
 *  \param s     start position within string resp. char array
 *  \param modTeleg pointer to the Modbus TCP/IP telegram
 *  \return points to the (0-) character behind the formated result
 */
char * formModTelegr(char * s, struct modTelegr_t * modTeleg);

/** @} */ // Doxygen addtogroup utilForm
//---------------------------------------------- arithmetic endianess  -----
/**
 * \addtogroup utilArith
 * @{
 *
 * @name Endianess Handling functions
 */
///@{

/** Toggle the endianess of a 32 bit value
 *
 *  \param value the value to convert
 *  \sa toggle16endian(ucnt16_t *)
 */
void toggle32endian(ucnt32_t * value);


/** Toggle the endianess of a 16 bit value
 *
 *  \param value the value to convert
 *  \sa toggle32endian(ucnt32_t *)
 *  \sa convert16endian
 */
void toggle16endian(ucnt16_t * value);


/** Convert the endianess of a 16 bit value
 *
 *  \param value the value to convert
 *  \return value with inverted endianess
 *  \sa toggle6endian
 */
uint16_t convert16endian(uint16_t value);

/** Convert the endianess of a 32 bit value
 *
 *  \param value the value to convert
 *  \return value with inverted endianess
 *  \sa toggle6endian
 */
uint32_t convert32endian(uint32_t value);


/** Add a normal (little endian) 16 bit value to 32 bit big endian
 *
 *  This type of addition is needed by \ref uip "uIP" and this is the
 *  "good" architecture specific implementation.
 *
 *  \param opRes the 32 bit big endian (network order) sum and summand
 *  \param op2   the 16 bit little endian summand
 *  \sa toggle32endian(ucnt32_t *)
 */
void add16littleTo32bigEndian(ucnt32_t * opRes, uint16_t op2);

///@} // end member group Endianess Handling functions

/** Set bit masks (table in flash memory)
 *
 *  This array of length 8 in flash memory contains the masks to set
 *  bits 0 .. 7 by ORing. Hence the values are 1, 2, 4 .... 128 (0x80).
 *  \sa clearBitMasks
 *  \sa setBitMask()
 */
  extern uint8_t const setBitMasks[8];

/** Clear bit masks (table in flash memory)
 *
 *  This array of length 8 in flash memory contains the masks to clear
 *  bits 0 .. 7 by ANDing. Hence the values are ~1, ~2, ~4 .... ~128 (0x7F).
 *  \sa setBitMasks
 *  \sa clearBitMask()
 */
  extern uint8_t const clearBitMasks[8];

 /** Get the 8 bit "Set bit mask" for the given bit
  *
  *  \param bitNum the bit number; the value will be taken modulo 8, i.e. 0..7
  *  \return the mask to set the given bit in a byte (by OR)
  *  \sa setBitMasks
  *  \sa clearBitMask()
  */
#define setBitMask(bitNum) (pgm_read_byte(&setBitMasks[bitNum&7]))


/** Get the 8 bit "Clear bit mask" for the given bit
 *
 *  \param bitNum the bit number; the value will be taken modulo 8, i.e. 0..7
 *  \return the mask to clear the given bit in a byte (by AND)
 *  \sa clearBitMasks
 *  \sa setBitMask()
 *
 *  This array of length 8 in flash memory contains the masks to set
 *  bits 0 .. 7 by ORing. Hence the values are 1, 2, 4 .... 128 (0x80).
 */
#define clearBitMask(bitNum) (pgm_read_byte(&setBitMasks[bitNum&7]))

/** @} */ // Doxygen addtogroup utilArith
//---------------------------------------------------    parsing     -------
/**
 * \addtogroup utilPars
 * @{
 */

/** Search the first token in a short (RAM) string
 *
 *  This function returns the start index \c (>=si) of the token found in
 *  string \c s.
 *  The begin of a token may be any non control code except space, comma,
 *  semicolon and equals ( ,;=). <br />
 *  255 is returned if no token was found before [len-1]. <br />
 *  This function does almost the same as \ref searchTokenStart() except that
 *  it skips over 0-codes. <br />
 *
 *  The rationale for this 0-code skipping is that \ref serCom "UART" input
 *  may get zeros from PC emulated terminals when this PC enters or leaves
 *  energy saving mode. Using  \ref searchTokenStart() after those situations
 *  would swallow the next command line entry. <br />
 *  The consequence is, of course, that parameter \c len must be correct. The
 *  too big \c len for 0-terminated string idiom fails with this function.
 *
 *  \param s   the RAM string to search in
 *  \param si  the index to start search (usually 0 for first token)
 *  \param len the maximum length of s to search in
 */
uint8_t searchFirstToken(char s[], uint8_t si, uint8_t len);


/** Search the next token in a short (RAM) string
 *
 *  This function returns the start index \c (>=si) of the token found in
 *  string \c s.
 *  The begin of a token may be any non control code except space, comma,
 *  semicolon and equals ( ,;=). <br />
 *  255 is returned if no token was found before [len-1] or a 0-code.
 *  This 0-code condition is the only difference to \ref searchFirstToken().
 *
 *  \param s   the RAM string to search in
 *  \param si  the index to start search
 *  \param len the maximum length of s to search in
 *  \sa searchTokenEnd
 */
uint8_t searchTokenStart(char s[], uint8_t si, uint8_t len);

/** Search the the end of a token in a short (RAM) string
 *
 *  This function returns the index of the first character following
 *  the the token starting at \c [si] in string \c s. It returns the \c len
 *  if no end was found before. The return value is \c <= \c len.
 *
 *  Any control character, space, comma, semicolon and equals ( ,;=) is
 *  considered as end of a token.
 *
 *  This function's return value (if < len) is a good start for the
 *  next token search by \code  searchTokenStart(s, retval, len) \endcode .
 *
 *  \param s   the RAM string to search in
 *  \param si  the (found) token's begin index
 *  \param len the maximum length of s to search in
 *  \sa searchTokenStart
 */
uint8_t searchTokenEnd(char s[], uint8_t si, uint8_t len);


/** Offset for token found in second list by \ref searchTokenIn
 *  \showinitializer
 */
#define INDEX_OFFSET_LIST2 125


/** Match a (short) token in a (RAM) string to a list of flash strings
 *
 *  This function returns the start index of the token's first match in
 *  a flash array \c list of flash strings. The array list must end with
 *  a NULL, like in example \code
 *  char* systemCommands[]) = {comHelp, comWDlong, NULL};
 *  \endcode
 *  The other entries, comHelp and comWDlong in the example, are 0-terminated
 *  strings in flash memory (PROGMEM).
 *
 *  The same applies to the second list \c list2. No list shall be longer
 *  than \ref INDEX_OFFSET_LIST2.
 *
 *  The return value is the first match's index (range 0 .. llen-1) or
 *  255 if no match could be found or is possible.
 *
 *  The match is made to the respective beginnings of the flash strings.
 *  A positive match will occur also if the flash string is longer. To
 *  avoid or recognise a hit to an ambiguous abbreviation a second search
 *  beyond the first hit is then made. 254 is returned if a second
 *  ambiguous hit is found.
 *
 *  The comparison is case insensitive. The token must not contain any
 *  control or white space characters.
 *
 *  \param s   the RAM string containing the token to search for
 *  \param si  the token's start index in s
 *  \param se  the token's end index + 1 in s
 *  \param list the flash array of flash strings to match to
 *              (ending with NULL)
 *  \param list2 the second flash array of flash strings to match to
 *              (ending with NULL) 120 is added to the return match index
 *              to distinguish the result from a match in \c list
 *
 *  \return    the index of a match in list; or the index +
 *             \ref INDEX_OFFSET_LIST2 of a match in list 2;
 *             or 255 if no match; or 254 if the token
 *             in s was too short to match unambiguously
 *  \sa searchTokenStart
 *  \sa searchTokenEnd
 */
uint8_t searchTokenIn(char s[], uint8_t si, uint8_t se,
                      char const * const * list, char const * const *  list2);


/** Copy the first SVN token or a full string from program space
 *
 *  This function copies the first SVN-token &mdash; i.e. Subversion
 *  keyword's (first) value &mdash;  from \c src to \c dest plus a trailing 0.
 *  Returned is the address of the last character in \c dest modified (i.e.
 *  the 0). In the example<br />
 *  "$Dte: 2012-12-19 14:13:57 +0100 (Mi, 19 Dez 2012) $"<br />
 *  the first SVN token would be "2012-12-19".
 *
 *  The operation will stop after \c mxLen characters transferred resp.
 *  modified. The \c mxLen count includes the 0 appended.<br />
 *  If \c src does not begin with a $ \c src will be transferred
 *  from start to first $ or white space (or end).
 *
 *  \param dest the destination to modify (in RAM)
 *  \param src the source (the SVN tag) to copy from (in flash memory)
 *  \param mxLen the maximum number of characters to modify in \c dest
 *         including the trailing 0 appended. The maximum advance of the
 *         return value to parameter \c dest
 *  \return dest + number of characters modified; NULL only if dest is NULL
 */
char * getFirstSVNtokenP(char * dest, char const * src, uint8_t mxLen);


/** Recognise one digit
 *
 *  This helper function returns the digit value 0..9 for characters '0'..'9'
 *  and 10..15 for characters 'a'..'f' or 'A'..'F'. If the parameter c is in
 *  neither range 16 is returned. Hence result &amp; 16 means invalid.
 *
 *  \param c   the character
 *  \return    0..10..15 if c is a valid decimal or hexadecimal digit
 *             and 16 otherwise
 */
uint8_t asDigit(uint8_t c)  __attribute__((always_inline));


/** Parse a two digit hexadecimal number
 *
 *  This function tries to interpret a part of the string \c s starting at
 *  \c si as one or two digit hexadecimal number. Returned is the index of
 *  the first character after that number in case of success and 0 otherwise.
 *
 *  Leading blanks will be skipped.
 *
 *  \param res pointer to the result; changed only if a hexadecimal
 *             number was parsed successfully
 *  \param s   the RAM string containing the 2 digit hex (00..FF/ff)
 *  \param si  the first digit's index in s
 *  \return    the index of the first character after the parsed number
 *             in case of success or 0 else
 *  \sa parseByteNum()
 *  \sa  asDigit()
 */
uint8_t parse2hex(uint8_t * res, char s[], uint8_t si);

/** Parse a one byte number
 *
 *  This function tries to interpret a part of the string \c s starting at
 *  \c si as an eight bit (one byte) number. Returned is the index of the
 *  first character after that (parsed) number in case of success and 0
 *  otherwise.
 *
 *  Leading blanks will be skipped. One sign character '+' or '-' is
 *  accepted. A minus will complement the result, notwithstanding it being
 *  declared unsigned or eventually parsed as hex. Leading zeroes don't, of
 *  course change, the result and they do not lead to octal interpretation.
 *
 *  A leading 0x or 0X will have the following one or two characters be
 *  interpreted as hexadecimal digits not counting leading zeros after the
 *  'x'. The parsing then will stop at the latest after the second digit
 *  even if the character following (where the returned value points to) is
 *  a legal hex digit character.
 *
 *  \param res pointer to the result; changed only if a one byte number
 *             (0..255 or 0x0 .. 0xFF) was parsed successfully
 *  \param s   the RAM string containing the digit(s)
 *  \param si  the first digit's index in s
 *  \return    the index of the first character after the parsed number
 *             in case of success or 0 else
 *  \sa parseWordNum()
 *  \sa parseDWordNum()
 *  \sa parse2hex()
 */
uint8_t parseByteNum(uint8_t * res, char s[], uint8_t si);


/** Parse a two byte number
 *
 *  This function tries to interpret a part of the string \c s starting at
 *  \c si as a 16 bit (type uint16_t) number. Returned is the index of the
 *  first character after that (parsed) number in case of success and 0
 *  otherwise.
 *
 *  Leading blanks will be skipped. One sign character '+' or '-' is
 *  accepted. A minus will complement the result, notwithstanding it being
 *  declared unsigned or eventually parsed as hex. Leading zeroes don't, of
 *  course change, the result and they do not lead to octal interpretation.
 *
 *  A leading 0x or 0X will have the following one to four characters be
 *  interpreted as hexadecimal digits not counting leading zeros after the
 *  'x'. The parsing then will stop at the latest after the forth digit even
 *  if the following character (where the returned value points to) is a
 *  legal hex digit character.
 *
 *  \param res pointer to the result; changed only if a number was
 *             parsed successfully
 *  \param s   the RAM string containing the digit(s)
 *  \param si  the first digit's index in s
 *  \return    the index of the first character after the parsed number
 *             in case of success or 0 else
 *  \sa parseByteNum
 *  \sa parseDWordNum
 */
uint8_t parseWordNum(uint16_t * res, char s[], uint8_t si);

/** Parse a four byte number
 *
 *  This function tries to interpret a part of the string \c s starting at
 *  \c si as a 32 bit (type uint32_t) number. Returned is the index of the
 *  first character after that (parsed) number in case of success and 0
 *  otherwise.
 *
 *  Leading blanks will be skipped. One sign character '+' or '-' is
 *  accepted. A minus will complement the result, notwithstanding it being
 *  declared unsigned or eventually parsed as hex. Leading zeroes don't, of
 *  course change, the result and they do not lead to octal interpretation.
 *
 *  A leading 0x or 0X will have the following one to eight characters be
 *  interpreted as hexadecimal digits not counting leading zeros after the
 *  'x'. The parsing then will stop at the latest after the eighth digit even
 *  if the character following (where the returned value points to) is a
 *  legal hex digit character.
 *
 *  \param res pointer to the result; changed only if a number was
 *             parsed successfully
 *  \param s   the RAM string containing the digit(s)
 *  \param si  the first digit's index in s
 *  \return    the index of the first character after the parsed number
 *             in case of success or 0 else
 *  \sa parseByteNum
 *  \sa parseWordNum
 */
uint8_t parseDWordNum(uint32_t * res, char s[], uint8_t si);

/** Check if a character is contained in a RAM string
 *
 *  This function return true if the character \c c is not 0 and contained
 *  in the 0-terminated string \c s.
 *
 *  \param s   the (0-terminated) RAM string to be searched for \c c
 *  \param c   the (non 0) character looked for
 *  \return    0 : \c c was not found in \c s, \c c : \c c was found in \c s
 */
uint8_t isContainedIn(char s[], const uint8_t c);

/** Check if a character is contained in a flash string
 *
 *  This function return true if the character \c c is not 0 and contained
 *  in the 0-terminated string \c s. \c s must be in flash memory.

 *  \param s   the (0-terminated) string in flash / program memory to be searched for \c c
 *  \param c   the (non 0) character looked for
 *  \return    0 : \c c was not found in \c s, \c c : \c c was found in \c s
 */
uint8_t isContainedIn_P(char s[], const uint8_t c);

/** @} */ // Doxygen addtogroup utilPars

#endif /* UTILS_H_ */
/** @} */ // Doxygen defgroup utilHelp
/** @} */ // Doxygen defgroup weAutSys
